let oldBorder = "";
let shouldAct = false;
let clicks = [];
let tabID = 0;

/**
 * Allows an element selection
 * @param event The event received when the cursor hovers over an element
 */
function mouseIn(event){
	if (!shouldAct) return;
	oldBorder = event.currentTarget.style.border;
	event.currentTarget.style.border = '1px dashed cyan';
	event.stopPropagation();
	event.preventDefault();
}

/**
 * Allows an element deselection
 * @param event The event received when the cursor hovers away from an element
 */
function mouseOut(event){
	if (!shouldAct) return;
	event.currentTarget.style.border = oldBorder;
}

/**
 * Evaluates an XPath
 * @param element The element to evaluate
 * @param forceNoID Should we force ignore flag
 * @returns Array
 */
function getPathTo(element, forceNoID = false) {
	if (element.id !== '' && !forceNoID)
		return ['id("' + element.id + '")', true];
	if (element === document.body)
		return [element.tagName, false];

	let ix = 0;
	let siblings = element.parentNode.childNodes;
	for (let i = 0; i < siblings.length; i++) {
		let sibling = siblings[i];
		if (sibling === element) {
			let data = getPathTo(element.parentNode, forceNoID);
			return [data[0] + '/' + element.tagName + '[' + (ix + 1) + ']', data[1]];
		}
		if (sibling.nodeType === 1 && sibling.tagName === element.tagName)
			ix++;
	}
}

/**
 * Allows an element to be selected
 * @param event
 */
function clicked(event){
	if (!shouldAct) return;

	let firstXPath = getPathTo(event.currentTarget)[0];
	let secondXPath = getPathTo(event.currentTarget, true)[0];
	let name = prompt("Comment voulez-vous nommer cet élément ?");
	clicks.push({xpath: firstXPath, xpath_fallback: secondXPath, name: name});

	mouseOut(event);
	shouldAct = false;
	browser.runtime.sendMessage({id: tabID, path: "icons/border-48.png"});
	event.stopPropagation();
	event.preventDefault();
}

/**
 * Master JQuery handlers
 */
$(document).on('mouseenter',"*", mouseIn);
$(document).on('mouseleave',"*", mouseOut);
$(document).on('click', "*", clicked);
$(document).on('mousedown', "*", function(event){
	if (!shouldAct) return;
	event.preventDefault();
});

/**
 * Enables or disables the handler
 */
function notify(data){
	console.log(data);
	if (data.type === 'export'){
		navigator.clipboard.writeText(JSON.stringify(clicks)).then(() => {
			alert("Données Inari Express exportées dans le presse-papiers.");
		});
	} else if (data.type === 'toggle'){
		shouldAct = !shouldAct;
		tabID = data.id; // For later use ?
		if (shouldAct){
			browser.runtime.sendMessage({id: tabID, path: "icons/border-48-red.png"});
		} else {
			browser.runtime.sendMessage({id: tabID, path: "icons/border-48.png"});
		}
	}
}

browser.runtime.onMessage.addListener(notify);