from os import getenv as env

import zmq
from loguru import logger


# This class allows for interprocess and intermachine discussion (1-1)
class Intercom:
	context: zmq.Context = None
	socket: zmq.Socket = None
	to: str = None
	port: int = None

	def __init__(self, to: str, port: int, isREQ: bool):
		self.context = zmq.Context()
		self.to = to
		self.port = port
		# noinspection PyUnresolvedReferences
		# zmq.REQ is defined at runtime, PyLint is therefore screaming. Disabled with the comment above
		self.socket = self.context.socket(zmq.REQ if isREQ else zmq.REP)
		if isREQ:
			self.socket.connect("tcp://{0}:{1}".format(to, port))
		else:
			self.socket.bind("tcp://{0}:{1}".format(to, port))

	# Sends a message
	def talk(self, what, isJSON: bool = True):
		logger.debug("Sending message to {0}...".format(self.to))
		if isJSON:
			self.socket.send_json(what)
		else:
			self.socket.send(what)
		logger.debug("Message sent.")

	# Listen for a message
	def listen(self, timeout=-1):
		logger.debug("Waiting for message...")
		if timeout > -1:
			# noinspection PyUnresolvedReferences
			self.socket.setsockopt(zmq.RCVTIMEO, timeout)
			# noinspection PyUnresolvedReferences
			self.socket.setsockopt(zmq.LINGER, 0)

		message = self.socket.recv_json()
		logger.debug("Message received")
		return message

	def ok(self):
		self.talk({"status": "OK"})

	def ko(self):
		self.talk({"status": "KO"})

	@staticmethod
	def broadcastToClients(clients, what):
		for client in clients:
			i = Intercom(client, int(env("CLIENT_PORT")), True)
			i.talk(what)
			i.listen()
