import multiprocessing
import requests
from loguru import logger
from os import getenv as env

from common.tasks.Runner import Runner


def setup(event):
	global unpaused
	unpaused = event


def work(data):
	context = {}

	wid = data['id']
	wargs = data['wargs']
	tasks = data['tasks']
	try:
		Runner(tasks, wargs).run()
		requests.post(url="http://" + env('SERVER_WEB_FQDN') + "/api/job/" + str(wid) + "/finished"
		              , verify=False)
	except Exception as e:
		logger.error("Stopped workflow because of an error.")
		logger.error(e)
		try:
			requests.post(url="http://" + env('SERVER_WEB_FQDN') + "/api/job/" + str(wid) + "/failed"
			              , verify=False)
		except Exception as e:
			pass

	logger.info("Done, returning context")

	return context


class Schnell:
	nWorkers: int = 0
	pool: multiprocessing.Pool = None
	chief: multiprocessing.Event = None

	def __init__(self, nWorkers: int):
		self.nWorkers = nWorkers
		self.chief = multiprocessing.Event()

	def start(self):
		logger.info("Starting pool with {0} workers...".format(self.nWorkers))
		self.pool = multiprocessing.get_context("spawn").Pool(self.nWorkers, setup, (self.chief, ))
		self.chief.set()

	# Stops the schnell pool.
	def stop(self):
		logger.warning("Stopping schnell pool...")
		self.pool.terminate()
		logger.warning("Done.")

	def pause(self):
		self.chief.clear()

	def resume(self):
		self.chief.set()
		logger.success("Unfrozen.")
