import random

from loguru import logger
from os import getenv as env
from common.tasks.Exceptions import XPathNotFoundException
from common.tasks.Runnable import Runnable


class SelectElement(Runnable):
	def run(self, parameters: dict) -> dict:
		target = parameters['xpath']
		target_fallback = parameters['xpath_fallback']
		instance = parameters['selenium_instance']

		if len(instance.find_elements_by_xpath(target)) == 0:
			logger.warning("XPath {} not found ! Falling back to secondary xpath.".format(target))
			if parameters['allowMissing']:
				return {'instance': instance, 'element': False}
			if len(instance.find_elements_by_xpath(target_fallback)) == 0:
				logger.error("Cannot find any of current XPath {}. The workflow mustn't be done properly."
				             .format(target_fallback))

				name = "oops-{0}.png".format(random.randint(0, 10000000))
				instance.save_screenshot(env("FILE_LOCKER") + name)
				raise XPathNotFoundException()
			else:
				element = instance.find_element_by_xpath(target_fallback)
		else:
			element = instance.find_element_by_xpath(target)

		return {'instance': instance, 'element': element}
