import os
import platform
import random

from random_user_agent.params import SoftwareName, OperatingSystem
from random_user_agent.user_agent import UserAgent
from os import getenv as env

from selenium import webdriver
from selenium.webdriver import FirefoxProfile

from common.tasks.Runnable import Runnable


class StartBrowser(Runnable):
	def run(self, parameters: dict) -> dict:
		# Setting variables
		exec_ext = "-mac" if platform.system() == "Darwin" else "-lin"
		executable = env("RESOURCES_PATH") + "bin/geckodriver" + exec_ext
		options = self.getBasicOptions()
		profile = self.getBasicProfile(parameters['useProxy'])

		# Running
		os.environ["PATH"] += os.pathsep + env("RESOURCES_PATH") + "bin"

		driver = webdriver.Firefox(executable_path=executable, options=options, firefox_profile=profile)

		return {'instance': driver}

	# Gets a random user agent for a PC
	def getRandomUserAgent(self):
		software_names = [SoftwareName.CHROME.value]
		operating_systems = [OperatingSystem.WINDOWS.value, OperatingSystem.LINUX.value]

		user_agent_rotator = UserAgent(software_names=software_names, operating_systems=operating_systems,
		                               limit=100)

		# Get Random User Agent String.
		user_agent = user_agent_rotator.get_random_user_agent()

		return user_agent

	# Gets a basic
	def getBasicOptions(self):
		options = webdriver.FirefoxOptions()
		options.add_argument("--headless")
		options.add_argument('--disable-gpu')
		options.add_argument("--disable-notifications")
		options.add_argument('--no-sandbox')
		options.add_argument('--verbose')
		options.add_argument('--lang=fr')
		options.add_argument('--disable-web-security')
		options.add_argument('--allow-running-insecure-content')
		options.add_argument('user-agent=' + self.getRandomUserAgent())

		return options

	def getBasicProfile(self, useProxy: bool):
		profile = FirefoxProfile()
		profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/vnd.openxmlformats"
		                                                                 "-officedocument.spreadsheetml.sheet,"
		                                                                 "application/pdf")
		profile.set_preference("browser.download.manager.showWhenStarting", False)
		profile.set_preference("browser.download.useDownloadDir", True)
		profile.set_preference("browser.download.folderList", 2)
		profile.set_preference("browser.download.dir", env("FILE_LOCKER"))
		profile.set_preference("browser.download.panel.shown", False)
		profile.set_preference("browser.helperApps.alwaysAsk.force", False)
		profile.set_preference("intl.accept_languages", "fr")
		profile.set_preference("pdfjs.disabled", True)
		if useProxy:
			proxyList = open(os.getcwd() + "/proxy-list.txt")
			line: str = next(proxyList)
			for num, aline in enumerate(proxyList, 2):
				if random.randrange(num):
					continue
				line = aline

			data = line.strip().split(":")

			profile.set_preference("network.proxy.type", 1)
			profile.set_preference("network.proxy.ssl", data[0])
			profile.set_preference("network.proxy.ssl_port", int(data[1]))

		profile.update_preferences()
		return profile
