from loguru import logger
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

from common.tasks.Exceptions import XPathNotFoundException
from common.tasks.Runnable import Runnable


class WaitForElementDisparition(Runnable):
	def run(self, parameters: dict) -> dict:
		instance = parameters['selenium_instance']
		target = parameters['xpath']
		target_fallback = parameters['xpath_fallback']

		if len(instance.find_elements_by_xpath(target)) == 0:
			logger.warning("XPath not found ! Falling back to secondary xpath.")
			if len(instance.find_elements_by_xpath(target_fallback)) == 0:
				logger.error("Cannot find any of current XPath. The workflow mustn't be done properly.")
				raise XPathNotFoundException()
			else:
				element = target_fallback
		else:
			element = target

		if instance.find_element_by_xpath(element).is_displayed():
			WebDriverWait(instance, 10).until(EC.invisibility_of_element_located((By.XPATH, element)))

		return {'instance': instance}
