from loguru import logger
from selenium.webdriver.firefox.webdriver import WebDriver


class Runner:
	tasks: list = []
	arguments: list = []

	def __init__(self, tasks, arguments):
		self.tasks = tasks
		self.arguments = arguments

	def run(self, hasTriedbefore=False):
		stopContext = []  # Instances of web browser will be stored here to close them in case of an exception
		try:
			context = {}
			for taskGroup in self.tasks:
				# Iterating on each
				for task in taskGroup:
					localTaskContext = task.run(context, self.arguments)
					for contextEntry in list(localTaskContext.values()):
						if isinstance(contextEntry, WebDriver):
							stopContext.append(contextEntry)

					context[task.task_uuid] = localTaskContext
			logger.info("Done, returning context")
			return context
		except Exception as e:
			logger.warning("Dumping stop context")
			for v in stopContext:
				v.quit()

			logger.error(e)
			if not hasTriedbefore:
				logger.info("Retrying")
				self.run(hasTriedbefore=True)
			else:
				logger.warning("Failed, again !")
				raise e
