from loguru import logger

from common.tasks.Basic.Argument import Argument
from common.tasks.Basic.MergeStrings import MergeStrings
from common.tasks.Basic.RandomNumber import RandomNumber
from common.tasks.Basic.ReplaceString import ReplaceString
from common.tasks.Basic.ToString import ToString
from common.tasks.Basic.Wait import Wait
from common.tasks.Browser.ClickElement import ClickElement
from common.tasks.Browser.FillComboBox import FillComboBox
from common.tasks.Browser.FillTextBox import FillTextBox
from common.tasks.Browser.SelectElement import SelectElement
from common.tasks.Browser.DownloadFilesFromBrowser import DownloadFilesFromBrowser
from common.tasks.Browser.EvaluateXPath import EvaluateXPath
from common.tasks.Browser.GetAttributeFromXPathNodes import GetAttributeFromXPathNodes
from common.tasks.Browser.GoToURLWithBrowser import GoToURLWithBrowser
from common.tasks.Browser.StartBrowser import StartBrowser
from common.tasks.Complex.ExportValuesToCSV import ExportValuesToCSV
from common.tasks.Complex.ExtractXLSXValues import ExtractXLSXValues


# Evaluates a Task's parameter map.
from common.tasks.Browser.StopBrowser import StopBrowser
from common.tasks.Browser.WaitForElementDisparition import WaitForElementDisparition
from common.tasks.Browser.WaitForURL import WaitForURL
from common.tasks.Complex.ReplaceDataFrameValues import ReplaceDataFrameValues
from common.tasks.Composer.GenerateList import GenerateList
from common.tasks.Composer.Multiplexer import Multiplexer
from common.tasks.Math.Addition import Addition
from common.tasks.Math.Maximum import Maximum
from common.tasks.Math.Multiply import Multiply
from common.tasks.Math.Substraction import Substraction
from common.tasks.Misc.ConvertIntToMonth import ConvertIntToMonth


def evaluateParametersWithContext(parameters: dict, context: dict):
	output = {}
	for key, value in parameters.items():
		if str(value).startswith("$"):  # Unpack parameter detected
			argumentRawPointer = value.replace("$", "").split(".")
			uuid = argumentRawPointer[0]
			ptrKey = argumentRawPointer[1]
			if uuid in context:
				output[key] = context[uuid][ptrKey]
			else:
				logger.error("Cannot replace contextual expansion with uuid {} as it does not exist within the context"
				             .format(uuid))
		else:
			output[key] = value

	return output


# Starts a task.
def startTask(taskID: int, parameters: dict, context: dict, arguments: dict):
	logger.info("Starting task ID {}...".format(taskID))
	parameters = evaluateParametersWithContext(parameters, context)
	if taskID == 1:
		return {"output": int(parameters["Valeur"]), "output2": int(parameters["Valeur"])}
	elif taskID == 2:
		return {"output": str(parameters["Valeur"]), "output2": str(parameters["Valeur"]),
		        "output3": str(parameters["Valeur"])}
	elif taskID == 3:
		return {"output": parameters["Valeur"], "output2": parameters["Valeur"]}
	elif taskID == 4:
		return StartBrowser().run(parameters)
	elif taskID == 5:
		return GoToURLWithBrowser().run(parameters)
	elif taskID == 6:
		return ReplaceString().run(parameters)
	elif taskID == 7:
		return EvaluateXPath().run(parameters)
	elif taskID == 8:
		return GetAttributeFromXPathNodes().run(parameters)
	elif taskID == 9:
		return DownloadFilesFromBrowser().run(parameters)
	elif taskID == 10:
		return StopBrowser().run(parameters)
	elif taskID == 11:
		# This is a dead end purposefuly made to stop the program
		return {}
	elif taskID == 12:
		return GenerateList().run(parameters)
	elif taskID == 13:
		return MergeStrings().run(parameters)
	elif taskID == 14:
		return SelectElement().run(parameters)
	elif taskID == 15:
		return FillComboBox().run(parameters)
	elif taskID == 16:
		return ClickElement().run(parameters)
	elif taskID == 17:
		return WaitForElementDisparition().run(parameters)
	elif taskID == 18:
		return FillTextBox().run(parameters)
	elif taskID == 19:
		return WaitForURL().run(parameters)
	elif taskID == 20:
		return ExtractXLSXValues().run(parameters)
	elif taskID == 21:
		return ReplaceDataFrameValues().run(parameters)
	elif taskID == 22:
		return ExportValuesToCSV().run(parameters)
	elif taskID == 23:
		return Multiplexer().run(parameters)
	elif taskID == 24:
		return ConvertIntToMonth().run(parameters)
	elif taskID == 27:
		return Argument(arguments).run(parameters)  # WARNING !
	elif taskID == 28:
		return Wait().run(parameters)
	elif taskID == 29:
		return RandomNumber().run(parameters)
	elif taskID == 30:
		return Addition().run(parameters)
	elif taskID == 31:
		return Substraction().run(parameters)
	elif taskID == 32:
		return Multiply().run(parameters)
	elif taskID == 33:
		return Maximum().run(parameters)
	elif taskID == 34:
		return ToString().run(parameters)
	else:
		logger.error("Unknown Module ID.")
		return
