from common.tasks import Starter


class Task:
	task_id: int = 0
	task_uuid: str = ""

	# The parameters for the launch of the task
	params: dict = {}

	# Determines whether a Task is an argument or not
	isArgument: bool = False

	# Determines the previous tasks' results dependencies.
	dependencies: dict = {}

	# Inits the Task.
	def __init__(self, task_id, task_uuid, params):
		self.task_id = task_id
		self.task_uuid = task_uuid
		self.params = params
		self.dependencies = {}  # Because apparently without this line it's a common list to every Task object
		self.cleanParams()
		self.computeDependencies()

	def computeDependencies(self):
		for key, value in self.params.items():
			if "$" in str(value):  # That's a dependency
				depRaw = str(value).replace("$", "").split(".")
				self.dependencies[key] = {"uuid": depRaw[0], "argument": depRaw[1]}

	# Remove the useless data from the params dict
	def cleanParams(self):
		if 'id' in self.params:
			self.task_id = self.params['id']
			self.params.pop('id')
		if self.task_id == 27:  # Argument
			self.isArgument = True

	def apply(self, key, value):
		self.params[key] = value

	# Runs a Task and returns its results
	def run(self, context, arguments):
		return Starter.startTask(self.task_id, self.params, context, arguments)

	# Quick conversion
	def toJSON(self):
		return {"params": self.params, "dependencies": self.dependencies, "id": self.task_id, "uuid": self.task_uuid}

	@staticmethod
	def fromJSON(data):
		return Task(data['id'], data['uuid'], data['params'])


# Returns a Task from the yaml data
def fromYAML(uuid, data: dict):
	return Task(data['id'], uuid, data)
