import falcon
import gunicorn.app.base
from loguru import logger

from common.schnell.workers import Schnell
from server.api.DocumentationAPI import DocumentationAPI
from server.api.FileAPI import FileAPI
from server.api.PlaybookAPI import PlaybookAPI, ArgumentAPI
from server.api.ThrottleAPI import ThrottleAPI


class API(gunicorn.app.base.BaseApplication):
	def __init__(self, application, options=None):
		self.options = options or {}
		self.application = application
		super().__init__()

	def load_config(self):
		config = {key: value for key, value in self.options.items()
		          if key in self.cfg.settings and value is not None}
		for key, value in config.items():
			self.cfg.set(key.lower(), value)

	def load(self):
		return self.application


def start(schnell: Schnell):
	options = {
		'bind': '%s:%s' % ('127.0.0.1', '8080'),
		'workers': 1,
	}
	logger.info("Started inarid with {} worker.".format(options['workers']))

	app = falcon.API()
	app.add_route('/', DocumentationAPI())
	app.add_route('/file', FileAPI())

	api = PlaybookAPI()
	api.setSchnell(schnell)
	app.add_route('/playbook', api)
	app.add_route('/playbookArgs', ArgumentAPI())

	app.add_route("/throttle", ThrottleAPI())

	API(app, options).run()
