<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\BuildRequest;
use App\Jobs\BuildPlaybook;
use App\Models\Build;
use App\Models\Workflow;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\ValidationException;
use Storage;

class BuildController extends Controller {

    public function __construct(){
        $this->middleware('auth:api');
    }

    /**
     * Display a listing of the resource.
     *
     * @param Workflow $workflow
     * @return \Illuminate\Http\Response
     */
    public function index(){
        return Auth::user()->builds()->with('workflow')->orderBy('created_at', 'DESC')->get();
    }

    /**
     * Shows the playbook if it's done.
     * @param Build $build
     * @return \Illuminate\Http\RedirectResponse|string
     */
    public function show(Build $build){
        if ($build->isFinished()){
            return response()->download(storage_path().'/app/builds/build-'.$build->workflow->id.'.yaml');
        } else {
            return back()->withErrors(['build_id' => "Ce build n'est pas terminé."]);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param BuildRequest $request
     * @return void
     * @throws ValidationException When the user has already sent a build request. Naughty user !
     */
    public function store(BuildRequest $request){
        if (Build::whereWorkflowId($request->workflow_id)->where('state', '<', 2)->count() > 0){
            throw ValidationException::withMessages(['workflow_id' => 'You cannot start two builds at the same time']);
        } else {
            $build = Build::create(['user_id' => $request->user()->id, 'workflow_id' => $request->workflow_id]);
            BuildPlaybook::dispatch($build);
        }
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\Build $build
     * @return void
     * @throws \Exception
     */
    public function destroy(Build $build){
        $build->delete();
    }
}
