<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\ClickRequest;
use App\Models\Click;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class ClickController extends Controller {

    public function __construct() {
        $this->middleware('auth');
    }

    /**
     * Display a listing of the resource.
     *
     * @return Click[]|\Illuminate\Database\Eloquent\Collection
     */
    public function index() {
        return Auth::user()->clicks;
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return void
     */
    public function store(ClickRequest $request) {
        $rawData = json_decode($request->data);
        foreach ($rawData as $click){
            Click::create([
                'xpath'             => $click->xpath,
                'xpath_fallback'    => $click->xpath_fallback,
                'name'              => $click->name,
                'user_id'           => $request->user()->id
            ]);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param Click $click
     * @return Click
     */
    public function show(Click $click) {
        return $click;
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Click $click
     * @return void
     * @throws \Exception
     */
    public function destroy(Click $click){
        $click->delete();
    }
}
