<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\JobRequest;
use App\Models\Client;
use App\Models\Job;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;

class JobController extends Controller {

    /**
     * Display a listing of the resource.
     *
     * @param Client $client
     * @return void
     */
    public function index($client) {
        try {
            $client = Client::findOrFail($client);
        } catch (ModelNotFoundException $e){
            $client = Client::whereIp($client)->firstOrFail();
        }

        return $client->jobs()->limit(1000)->get();
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param JobRequest $request
     * @param Client $client
     * @return void
     */
    public function store(JobRequest $request, $client) {
        try {
            $client = Client::findOrFail($client);
        } catch (ModelNotFoundException $e){
            $client = Client::whereIp($client)->firstOrFail();
        }

        return Job::create([
            'workflow_id' => $request->workflow_id,
            'data' => json_decode($request->params),
            'client_id' => $client->id
        ])->id;
    }

    /**
     * Display the specified resource.
     *
     * @param Job $job
     * @return Job
     */
    public function show(Job $job) {
        return $job;
    }

    /*
     * QUICK DOCUMENTATION ON STATUS
     * 0 : STANDING BY
     * 1 : FAILED
     * 2 : SUCCEEDED
     */

    /**
     * Marks a job as finished.
     * @param Job $job
     */
    public function markAsFinished(Job $job){
        $job->status = 2;
        $job->save();
    }

    /**
     * Marks a job as failed.
     * @param Job $job
     */
    public function markAsFailed(Job $job){
        $job->status = 1;
        $job->save();
    }

}
