<?php

namespace App\Http\Controllers;

use App\Models\Client;
use App\Models\Job;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Storage;

class JobController extends Controller {

    public function __construct(){
        $this->middleware('auth');
    }

    /**
     * Display a listing of the resource.
     *
     * @param Client $client
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function index($client) {
        try {
            $client = Client::findOrFail($client);
        } catch (ModelNotFoundException $e){
            $client = Client::whereIp($client)->firstOrFail();
        }

        return view('pages.job.job', ['client' => $client->load('jobs')]);
    }

    /**
     * Cleans all jobs for a specific client
     * @param Client $client The client
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     */
    public function cleanAll(Client $client){
        $client->jobs()->delete();
        return redirect(route('job.index', $client));
    }

    /**
     * Cleans all failed jobs for a specific client
     * @param Client $client The client
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     */
    public function cleanFailed(Client $client){
        $client->jobs()->whereStatus(1)->delete();
        return redirect(route('job.index', $client));
    }

    /**
     * Cleans all finished jobs for a specific client
     * @param Client $client The client
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     */
    public function cleanFinished(Client $client){
        $client->jobs()->whereStatus(2)->delete();
        return redirect(route('job.index', $client));
    }

    /**
     * Retries all failed tasks for a particular client
     * @param Client $client
     * @throws \Illuminate\Validation\ValidationException
     */
    public function resume(Client $client){
        $failed = $client->jobs()->with('workflow')->whereStatus(1)->get();
        $bridge = new \App\Http\Controllers\API\WorkflowController();

        $failed->each(function($job) use ($client, $bridge){
            $bridge->talkToClient($job->workflow, ['client' => $client->id]);
        });

        $client->jobs()->with('workflow')->whereStatus(1)->update(['status' => 0]);

        return back();
    }
}
