<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Collection;

class Module extends Model {
    protected $casts = ["arguments" => "object"];
    protected $fillable = ['user_id', 'name', 'description', 'arguments'];

    /**
     * Returns a YAML compatible
     * @param $moduleInfo Collection The module info
     * @return array
     */
    public function toYAML($moduleInfo){
        $output = [
            'id' => $this->id,
        ];

        // Adding dependencies
        foreach ($moduleInfo->connections->inputs as $arg => $data){
            $output[$arg] = "$".$data->module.".".$data->connector."$";
        }
        // Adding actual current parameters
        foreach($moduleInfo->params as $arg => $data){
            // We support object passed in JSON. They will be flattened, though.
            // Warning : should both keys be named the same, the first one WILL be erased.
            if (is_object($data)){
                foreach($data as $oKey => $oData){
                    $output[$oKey] = $oData;
                }
            } else {
                $output[$arg] = $data;
            }
        }

        return $output;
    }
}
