<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::name('api.')->group(function(){
    Route::group(['namespace' => 'API'], function(){
        Route::group(['middleware' => 'auth:api'], function(){
            // User only services

            // Workflow related routes
            Route::resource('workflow', 'WorkflowController')->except(['edit', 'create']);
            Route::resource('workflow.module', 'WorkflowModuleController')->except(['edit', 'create']);

            // Builds the workflow (json => .py file)
            Route::resource("build", "BuildController")->only(['index', 'store', 'destroy', 'show']);

            // JSON representation of the workflow
            Route::post("workflow/{workflow}/json", "WorkflowController@storeJSON")
                ->name('workflow.storeJSON');
            Route::get("workflow/{workflow}/json", "WorkflowController@getJSON")
                ->name('workflow.getJSON');

            Route::post("workflow/{workflow}/arguments", "WorkflowController@getArguments")
                ->name('workflow.arguments');
            Route::post("workflow/{workflow}/run", "WorkflowController@run")->name('workflow.run');
            Route::post("workflow/{workflow}/bulk", "WorkflowController@bulk")->name('workflow.bulk');
            Route::post("workflow/{workflow}/runBulk", "WorkflowController@runBulk")
                ->name('workflow.runBulk');

            // Module in general. That is, modules without their user-bound representation
            Route::get('module/search', 'ModuleController@search')->name('module.search');
            Route::resource('module', 'ModuleController')->only(['index', 'store']);

            // Clicks
            Route::resource('click', 'ClickController')->only(['index', 'show', 'store', 'destroy']);

            Route::get("file/{client}", "FileController@showFiles")->name('client.showFiles');
        });

        // Unprotected resources
        // Jobs (linked to clients)
        Route::post('/job/{job}/finished', 'JobController@markAsFinished')
            ->name('job.markAsFinished');
        Route::post('job/{job}/failed', 'JobController@markAsFailed')
            ->name('job.markAsFailed');
        Route::resource('client.job', 'JobController')->only(['index', 'store']);

        // Clients
        Route::resource('client', 'ClientController')->only(['index', 'show', 'store', 'destroy']);
        Route::get("/client/{client}/ping", 'PingController@ping')->name('client.ping');
    });
});
