# Performances d'Inari

Dans cette partie, nous étudierons plusieurs exécutions d'Inari afin de déterminer son comportement, ses performances et les résultats produits.
Nous commencerons par analyser un lancement de la FAO Genève en configuration réduite, puis sur plusieurs coeurs, puis sur deux machines avec plusieurs coeurs.
Nous continuerons par analyser un lancement réduit sur Swiss-Impex, c'est-à-dire un lancement sur une seule catégorie de marchandises; nous terminerons ce chapitre par l'analyse d'un lancement total sur Swiss-Impex. Nous partirons du principe qu'il y aura toujours une base de données, un serveur et au moins un client fonctionnel en tout temps.

Le but de cette section est de répondre à deux questions :

* Est-ce que l'automatisation de la récupération des documents est plus rapide que le téléchargement des documents un par un ?
* Est-ce que la distribution des tâches de téléchargement accélère le traitement des tâches ?

L'ensemble des mesures de ce projet sont disponibles dans le dossier `projet/resources/measures`.

## Analyse FAO Genève

Nous allons nous intéresser au lancement de la récupération de $1000$ documents sur la FAO Genève. La configuration d'Inari sera faite de trois façons différentes :

* Une machine, un travailleur (ce scénario sera abrégé 1M1T, pour Une Machine Un Travailleur)
* Une machine, six travailleurs (ce scénario sera abrégé 1M6T)
* Deux machines, six travailleurs chacun (ce scénario sera abrégé 2M6T, où chaque machine a six travailleurs)

On utilise le _playbook_ présenté dans le fichier `projet/resources/playbooks/fao-geneve.yml` mis en annexe. Nous utilisons six coeurs : en effet, la majorité des machines aujourd'hui ont huit coeurs présents dans leur processeur. Utiliser six coeurs permet d'éviter de surcharger la machine (et donc de potentiellement ralentir le système).

### Théorie

Les documents sont au nombre de $1'000$. Si on part du principe que 50 documents sont sur une page, et que $50$ documents sont récupérés en une minute[^16], on peut poser :

\begin{center}
$\frac{N_{documents} * T_{50 documents}}{50} = \frac{1000 * 60}{50} = 1200$
\end{center}

Soit 20 minutes théoriques dans le cas où nous n'avons qu'une machine et un travailleur. Dans le cas où nous avons $n$ travailleurs, il faudra diviser la durée par $n$; dans le cas où nous avons $n$ travailleurs et $k$ machines (où chaque machine dans $k$ a exactement $n$ travailleurs), il faudra diviser la durée théorique par $k*n$.

[^16]: Vitesse supposée de téléchargement d'un humain ne faisant aucune faute et ne se fatiguant pas

### Mesures

Après envoi de la commande au-x client-s, on observe après chaque seconde le nombre de documents téléchargés au sein du dossier de chaque client. Les mesures seront effectuées quatre fois par scénario afin d'établir que nos mesures sont exactes.

### Résultats

Nous obtenons trois graphiques, soit un par scénario :

\begin{figure}
  \begin{subfigure}{\linewidth}
  \includegraphics[width=.5\linewidth]{figs/measures/1m1t.pdf}\hfill
  \includegraphics[width=.5\linewidth]{figs/measures/1m6t.pdf}\hfill
  \centering
  \includegraphics[width=.5\linewidth]{figs/measures/2m6t.pdf}
  \end{subfigure}\par\medskip
  \caption{Performances de la FAO Genève sur 1000 documents}
  \caption*{\textit{Source : réalisé par Théo Pirkl}}
\end{figure}

Comme nous pouvons le constater, nous avons largement dépassé la vitesse théorique d'un humain. En prenant le cas du premier graphique (c'est-à-dire le scénario 1M1T), on peut observer sur la courbe croissant le moins vite qu'on atteint une vitesse de traitement de données de 1.84 documents par seconde, soit une vitesse d'environ 110 documents par minute, soit deux fois plus vite qu'un être humain.
Les temps moyens sont :

* Dans le premier scénario on obtient comme temps $542$, $534$, $496$ et $398$ secondes, pour un temps moyen de $492.5$ secondes;
* Dans le second, les temps sont de $271$, $181$, $212$ et $258$ secondes, donnant en moyenne $230.5$ secondes;
* Le dernier scénario a les plus petites mesures de temps, de $71$, $46$, $54$ et $52$ secondes, pour un temps moyen de $55.75$ secondes.
Au vu des temps obtenus, il est évident que le scénario le plus efficace est le 2M6T.

On peut se donner une idée plus concrète de performance en convertissant les scores en pourcentage de performance. On prend le score moyen du scénario 1M1T comme base, et on divise ce score par les autres scores moyens obtenus, ce qui nous donnera un pourcentage de performance. Ce pourcentage sera par rapport à la performance du premier scénario. Cela donne :

* $\frac{492.5}{230.5} \approx 2.13$, soit environ $213\%$ d'efficacité en plus.
* $\frac{492.5}{55.75} \approx 8.83$, soit environ $883\%$ d'efficacité en plus.

On constate plusieurs choses. D'une part, le multiprocessing et la distribution des tâches apportent des performances accrues : $883\%$ est une amélioration considérable qui est la bienvenue si l'on traite de gros volumes de fichiers. Pour se donner un ordre d'idée, un humain a besoin de huit heures pour récupérer 24'000 documents (selon l'estimation de vitesse faite plus haut). En gagnant $883\%$ de performance, on réduit ce temps à un peu moins d'une heure.
D'autre part, on remarque que la fonction $Pourcentage_{performance}(Nombre_{coeurs})$ ne semble pas linéaire. Nous pouvons l'étudier pour se donner une idée de la courbe de gain de performance en fonction du nombre de coeurs.

\cimg{figs/measures/faogeneve-perf.pdf}{scale=.8}{Analyse de la fonction de gain de performance du module FAO Genève distribué}{Source : réalisé par Théo Pirkl}

Des fonctions usuelles nous donnent une idée de la croissance et du "type" de fonction que serait la fonction de gain de performance. Il semblerait que la courbe de gain de performances soit de plus en plus performante, un peu comme une exponentielle (bien qu'elle ne croit pas aussi vite qu'une véritable exponentielle).
Ces constats se basent sur les mesures faites et présentées plus haut : il est envisageable que la croissance de la fonction de gain explose un peu plus loin et que le constat que nous venons de faire soit caduc. Toujours est-il que notre gain ne semble pas linéaire.

### Bilan

Au vu de ce qui a été dit plus haut, nous pouvons répondre aux questions que nous avons posé de cette façon :

* L'automatisation du téléchargement des documents, dans le cas de la FAO Genève, est significativement plus rapide que le téléchargement des documents un à un;
* La distribution des tâches de téléchargement rend ce qui était déjà rapide encore plus rapide avec un gain, comme nous l'avons observé, allant jusqu'à $883\%$.

Le scraping semble donc extrêmement utile dans le cas de la FAO.

## Analyse Swiss-Impex réduite

Nous allons maintenant analyser le module Swiss-Impex. Ce module est plus intéressant à étudier que la FAO Genève : en effet, Swiss-Impex est un scraper de catégorie deux. Ces modules sont plus complets et complexes que ceux de première catégorie. La récupération des données prendra bien plus de temps que la FAO Genève.
Dans le cas d'une analyse Swiss-Impex réduite, on ne s'intéresse qu'à une seule catégorie de marchandises. Dans ce cas, on s'intéressera à la catégorie *2709.0090*, ce qui correspond aux données pétrolières Suisses.

On utilise le _playbook_ présenté dans le fichier `projet/resources/playbooks/swiss-impex-petroleum.yml`, mis comme première annexe.

### Théorie

Swiss-Impex propose un rapport d'import/export par mois depuis 1988 jusqu'à 2019 (les informations de 2020 étant encore relativement inexactes).
On peut donc calculer le nombre de documents à $12 * (2019 - 1989) = 360$.

En admettant que chaque document prenne une minute pour être téléchargé par un humain. Avec une seule machine ayant six workers, cela prendrait donc une heure. Pour réduire au plus ce temps, nous allons utiliser vingt machines. Chacune de ces machines aura un travailleur. Nous aurons donc un total de 20 travailleurs capables d'effectuer des tâches simultanément. Nous choisissons de ne pas mettre trop de travailleurs : en effet, trop de travailleurs déclenchent un déni de service accidentel sur le site, le rendant inopérant temporairement.
Nous passons donc de une heure à 18 minutes théoriques.

### Mesures

Après envoi de la commande aux clients, on pourrait observer toutes les 15 secondes[^17] le nombre de documents téléchargés (le total calculé sur toutes les machines). Toutefois, compte tenu du nombre de clients, nous allons plutôt observer le nombre d'entrées dans la base de données centrale. En effet, chaque document à traiter correspond à une entrée dans la base de données. Cela ne changera rien à nos mesures. Le graphique sera toutefois inversé : plutôt que de commencer à zéro document téléchargé, on commencera à 360 documents en attente de téléchargement. La courbe ne croîtra donc pas, mais descendra progressivement jusqu'à 0. Toutefois, cela ne nous empêche pas de calculer le pourcentage de gain d'efficacité.

[^17]: Observer plus rapidement que toutes les 15 secondes saturerait le protocole de communication : les mesures arriveraient tardivement et seraient décalées à la réalité. Nos mesures seraient alors biaisées.

### Résultats

Comme nous l'avons indiqué plus haut, les scrapers sont très sensibles aux changements de site. Si l'éditeur proposant le site n'a pas d'accord avec l'entité scrappant son site, elle peut du jour au lendemain mettre à jour son site et donc potentiellement rendre le scraper inutile. C'est ce qui est arrivé dans notre cas : au moment de la mesure, Swiss-Impex avait changé des informations permettant au moteur JavaScript de se repérer sur la page. Le scraper Swiss-Impex est alors devenu "aveugle" et ne pouvait plus traiter les documents proprement. La structure, l'apparence ou la logique interne du site n'avaient manifestement pas été changées : seuls les éléments permettant au scraper de se repérer avaient été changés.
Toujours au moment de la mesure, il était trop tard pour mettre à jour le scraper convenablement : il a donc été laissé en l'état.

\begin{figure}
  \begin{subfigure}{\linewidth}
  \includegraphics[width=.4\linewidth]{figs/swiss-impex-before.png}\hfill
  \includegraphics[width=.4\linewidth]{figs/swiss-impex-after.png}\hfill
  \end{subfigure}\par\medskip
  \caption{Changements effectués en date du 23.03.2020}
  \caption*{\textit{Source : réalisé par Théo Pirkl}}
\end{figure}

Des informations intéressantes ont toutefois pu être récoltées : il est nécessaire de limiter volontairement le nombre de travailleurs agissant en même temps sur un site, sans quoi l'on peut se retrouver banni. C'est ce qui arrivé : les ordinateurs de l'HEPIA n'ont pas été bannis, pourtant c'est eux qui ont généré le plus gros volume de données. Quant à l'ordinateur sur lequel ce projet a été conçu, il a été banni relativement rapidement (à ce jour, il n'a pas totalement été "débanni"). Cela laisse à penser que Swiss-Impex autorise toutes les adresses IPs étatiques à télécharger autant qu'elles le souhaitent. Enfin, au vu de l'échec de la récupération des données, de la difficulté à faire fonctionner proprement le scraper et des changements, il est possible que Swiss-Impex tente activement de bloquer les scrapers. Aucune preuve concrète n'est disponible, toutefois sur l'expérience de ce travail de semestre, cela reste une possibilité.

Il est impossible en l'état de déterminer un facteur d'efficacité concret, mais il est toutefois possible de consulter le fichier `resources/measures/tasks-perf.csv`, qui est la mesure de temps qu'a pris chaque tâche, mesure intégrée à toutes les tâches traitées par Inari. Ce temps n'est disponible que si la tâche a réussi : on peut donc considérer ces temps comme valides.
Dès lors, on peut donc au moins déterminer quelques informations.
Nous avons dans ce fichier 103 entrées : nous pouvons donc calculer un temps moyen nécessaire à la récupération d'un document. Ce temps moyen donne en l'occurrence $37.1 [s]$.
Si on applique ce temps moyen au nombre de documents que nous allons télécharger, on obtient un temps de $37 [s] * 360$, ce qui donne trois heures et 42 minutes avec un seul travailleur, et environ 11 minutes avec 20 travailleurs. Ce qui est de toute manière plus efficace qu'un humain (même à un seul travailleur) : si on garde la vitesse de récupération théorique d'un humain, on est $162\%$ plus efficace.

### Bilan

L'application d'Inari sur le site Swiss-Impex n'est pas un franc succès. Nous n'avons pas pu récupérer les informations du site d'une catégorie de marchandises : à partir de $100$ connexions (venant d'IPs différentes), le site s'arrête de fonctionner, ce qui fait planter le scraper. Il faut donc réduire le nombre de workers total pour pouvoir être en mesure de télécharger correctement les informations du site. Il est toutefois possible d'en tirer un grand nombre d'enseignements, qui pourront mener sur des fonctionnalités potentiellement implémentables pendant le travail de bachelor. Ce n'est donc pas une perte, loin de là puisqu'on est malgré tout capable, en plus des enseignements collectés, d'obtenir quelques informations en extrapolant les données à notre disposition.

On peut tenter de répondre aux questions que nous avons posées au début de ce chapitre :

* L'automatisation de la récupération des documents semble plus rapide que la récupération de documents un par un;
* La distribution semble aussi accélérer la récupération de documents. Si on se base toutefois seulement sur les mesures de temps citées plus haut, il est impossible ici de le prouver.

## Analyse Swiss-Impex totale

Nous allons finalement effectuer un téléchargement complet de Swiss-Impex, sur toutes les données possibles à télécharger.

### Théorie

Toujours dans l'optique qu'un utilisateur est capable de télécharger un document en une minute, commençons par calculer le nombre de documents théoriquement disponible sur Swiss-Impex.
Si on consulte la liste de toutes les catégories de marchandise disponibles dans `resources/merchandises/EUMerchIDs.txt` et qu'on considère les lignes contenant une marchandise précise (à savoir les lignes comportant quatre chiffres, un point, puis à nouveau quatre chiffres), on a un total de $6 837$ catégories. Cela représente un nombre important de tâches. Pour éviter les surcharges, ce fichier a été coupé en 100 plus petites parties, toutes disponibles dans le même dossier.

En partant du principe que chaque catégorie contient (comme les données pétrolières) $360$ documents, on arrive à un total de $2 461 320$ documents à télécharger. Nous ne pouvons pas démarrer trop de noeuds (ici de machines client) sans quoi le serveur distant va s'arrêter. Nous allons donc rester avec $20$ noeuds.

Cette section n'aura pas de partie mesures ou résultats ni de bilan : en effet, au vu de ce qui précède, nous ne pouvons pas récolter de mesures sur Swiss-Impex. On peut toutefois estimer le temps nécessaire total de récolte théorique en se basant sur la mesure de la section du dessus : $2461320 * 37 = 91 068 840 [s]$, soit 37 mois, c'est-à-dire la durée de plus de cinq projets de semestres mis bout à bout.
L'analyse Swiss-Impex totale reprend toutes les caractéristiques de l'analyse Swiss-Impex partielle, à la seule différence qu'elle est plus longue. Il semble donc redondant d'ajouter un bilan à cette partie : il est plus intéressant de faire un bilan global des résultats.

## Bilan global des résultats

Les résultats sont plutôt prometteurs. On observe, malgré la difficulté à en obtenir certaines, des mesures prouvant à quel point la machine peut aider l'homme à la récupération de données et à quel point il est facile de décupler la vitesse de récupération. Il est aussi possible d'apprendre des échecs lors de nos mesures : les scrapers ne sont pas évidents à maintenir, car aucun accord n'est véritablement établi entre l'auteur du site et l'utilisateur du scraper. Tout peut cesser de fonctionner du jour au lendemain, que cela soit à cause d'un changement du site web, de politique quant aux scrapers, d'un excès de trafic venant du scraper, ou à une maintenance.
Outre la difficulté, une augmentation observée sur les sites traités à un minimum de $200\%$ prouve à quel point les scrapers sont une catégorie précieuse de logiciels. Si le scraper fonctionne, les résultats peuvent s'avérer suffisamment intéressants pour passer outre la difficulté.

\pagebreak