# Étude des approches actuelles

Sans les connaissances nécessaires, un utilisateur s'aventurant dans la récupération de documents sur Internet est très vite confronté à la complexité de la matière et se résout, de manière générale, à télécharger les documents un par un.
La Suisse a lancé il y a quelques années une initiative [OpenData.swiss](https://opendata.swiss/fr/) qui en est, tel que WikiData, encore à l'étape de balbutiement et ne semble pas connue du grand public.

\cimg{figs/internet-minute.jpg}{scale=0.5}{Ce que produit Internet chaque minute (en 2019)}{Source : \url{www.visualcapitalist.com}, ref. URL01}

Il est de plus en plus urgent d'automatiser la récupération de données. Si Forbes estimait en 2018 la création de $2500000$ terabytes de données par jour [@marr_how_nodate], certains vont même jusqu'à tirer la sonnette d'alarme en déclarant qu'en 2025, 463 Exabytes seraient générés chaque jour [@desjardins_how_2019], ce qui représente une quantité inimaginable de données. La figure 1.1 donne une représentation plus tangible de la quantité de données envoyée chaque minute sur Internet. Le terme *data smog*, existant depuis 1997 [@shenk_data_1997], prend aujourd'hui de plus en plus de sens.
L'entreprise Nandex affirme qu'un employé perd 2.5 heures par jour à chercher des documents [@noi_workers_nodate]. Au vu de l'explosion de la quantité de données, c'est un temps lui aussi exponentiel que passera chacun et chacune à trouver les documents qu'il ou elle cherche.

La récupération des données automatisée est un moyen de réduire le temps perdu à chercher un document en déléguant la tâche à un automate, plus précisément à un logiciel informatique.

Plusieurs façons de télécharger les documents existent. Il est nécessaire de les étudier afin de comprendre leur fonctionnement et leurs implications.

Morale et éthique, faute de définition ou de référence objective sont des notions indéterminées. Ci-après, l'auteur entend par éthique, un comportement qui n'est pas illégal et qui n'a pas pour objet d'abuser des ressources d'autrui ou de les utiliser dans un but que leur auteur ne leur destinait pas.  

## Étude et analyse

Chaque site web contient un certain nombre de données : documents, images, films, présentations, texte. Tout contenu lisible par un utilisateur peut être considéré comme téléchargeable. Au vu de l'hétérogénéité des sites web en matière de style, de topologie et de façons de construire un site, aucun véritable standard n'existe sur les sites web ; chacun se sert des outils permettant de faire son site comme bon lui semble. Il n'existe dès lors aucun standard pour récupérer des données. La figure 1.2 donne quelques exemples de sites web possédant une structure, un style et des données différentes.

\begin{figure}[t!]
    \begin{subfigure}{.48\linewidth}
        \includegraphics[width=\linewidth]{figs/website-1.png}
    \end{subfigure}\hspace*{\fill}
    \begin{subfigure}{.48\linewidth}
        \includegraphics[width=\linewidth]{figs/website-2.png}
    \end{subfigure}\medskip

    \begin{subfigure}{\linewidth}
        \centering
        \includegraphics[width=.6\linewidth]{figs/website-3.png}
    \end{subfigure}\hspace*{\fill}

    \caption{Comparaison du style, topologie et façon de faire de trois sites web} \label{fig:1}
    \caption*{\textit{Source : \url{www.ranch.ch}, \url{www.archive.org}, \url{www.admin.ch}, ref. URL02, URL03, URL04}}
\end{figure}

Il existe toutefois plusieurs techniques particulièrement utilisées. Certaines entités proposent un service permettant de récupérer leurs données dans un format directement extractible : on parle alors de manière générale d'une (+API_a). D'autres proposent de diffuser l'information par des flux (+RSS_a), l'équivalent d'un blog lisible par un ordinateur. Certains proposent encore, malgré leur disparition, d'utiliser des CDs, DVDs ou disque durs pour récupérer leurs données, ce qui est le cas d'un des services de l'État de Genève [@noauthor_commande_nodate].

Au vu de leur utilisation, les (+API_a) restent aujourd'hui la méthode préférée d'échange de données. Elles ont l'avantage, contrairement aux autres méthodes, de proposer quasi-directement la récupération des informations.

Les (+API_a) cependant restent rares; de nombreux sites n'en ont pas. Dans le cas où elles en proposent, elles sont souvent payantes, accessibles sur permission uniquement, limitées et peu ou pas documentées.
En outre, les (+API_a), lesquelles représentent $83\%$ du trafic HTTP mondial [@akamai_state_2019], constituent une lourde charge pour l'exploitant du site, qu'elle soit technique ou financière. Qui plus est, il est potentiellement nécessaire de payer un développeur pour la développer et la maintenir.

D'autres sites, qui composent la grande majorité des sites, ne proposent aucun système de récupération. Le site web lui-même est le seul accès aux documents possible. L'utilisateur doit alors aller récupérer manuellement les documents un par un. Par exemple, un des sites de l'Etat de Genève nommé FAO refuse de proposer une API car les informations sont disponibles gratuitement en ligne [@departement_presidentiel_notitle_2018].

Il existe une méthode alternative nommée _scraping_. Cette dernière consiste à simuler la visite d'un site, comme si un utilisateur le visualisait, pour récupérer les informations s'y trouvant.
Cette méthode, très utilisée par les géants de l'Internet comme Google [@kosir_is_2014], n'est pas particulièrement utilisée ailleurs. Les logiciels de scraping disponibles sur le marché sont rares et sont répartis en trois catégories détaillées plus bas. Beaucoup d'entreprises considèrent cette pratique comme malveillante et n'hésitent pas à attaquer toute personne ou entreprise connue pour avoir utilisé une méthode de scraping sur leur site. C'est toutefois une pratique qui n'est pas illégale, puisque la justice a, à plusieurs reprises, donné raison à des entreprises accusées de scraper des sites tiers [@bode_court_2019] [@igor_us_2018]
En outre, les scrapers (les logiciels en charge de l'opération de _scraping_) ont la réputation d'être fragiles, ces derniers se basant sur l'architecture du site scrapé. Le moindre changement dudit site provoque de manière générale l'arrêt du fonctionnement du scraper.

Trois grandes catégories de scrapers existent :

* Les solutions de type framework [^2];
* Les solutions payantes, nommées (+SAAS_a);
* Les solutions payantes humaines.

[^2]: Un framework est un ensemble de composant permettant de poser les fondations d'un logiciel de façon à ne pas tout programmer de zéro. [@noauthor_framework_nodate]

## Comparaison des approches existantes

Trois solutions existent aujourd'hui. Afin de pouvoir comprendre les subtilités de chacune des trois catégories, il est nécessaire de faire une rapide analyse.

### Comparaison technique

Les _framework_ sont, de manière générale, offerts sans support et demandent des connaissances en informatique relativement poussées. Ces derniers sont proposés sans support et s'avèrent parfois complexes. Cependant, en acceptant de passer les difficultés, un framework permet par la suite de faire absolument ce que l'on veut. Par exemple, une fonctionnalité manquante peut tout à fait être ajoutée par un utilisateur. Le framework peut être rendu plus robuste, plus sécurisé ou plus simple grâce à un de ces utilisateurs.

Les plateformes (+SAAS_a) possèdent les caractéristiques inverses. Elles sont payantes, possèdent un support et demandent moins de connaissances. La difficulté pour se servir d'un (+SAAS_a) est moindre comparé à un framework. Toutefois, les (+SAAS_a) supportent un éventail de fonctionnalités bien défini, qui est difficile voire impossible d'étendre.

La dernière catégorie, les solutions payantes humaines, consiste à payer une tierce personne pour qu'elle récupère les documents. Cette solution est analysée en détail plus bas.

Un autre point de vue à considérer est la partie _Open Source_ :  si le logiciel est Open Source, il est possible de consulter son code pour l'améliorer, ajouter des fonctionnalités et s'assurer qu'il fonctionne exactement conformément à ses spécificités. Les logiciels non open source sont décrits comme _privateurs de liberté_ par de nombreux développeurs célèbres comme Richard Stallmann [@williams_hackers_2002]. Il est impossible de dire avec certitude ce que fait un logiciel qui n'est pas open source, justement parce que le code source n'est pas disponible.

### Comparaison morale

La comparaison morale est un point qui peut sembler peu important au premier abord pour la questiom du scraping, mais qui est en vérité  essentiel lors de l'étude des différentes solutions.
En considérant certaines solutions existantes sur le marché, comme la plateforme _Mechanical Turk_ d'Amazon, il est possible d'y apercevoir, selon l'opinion publique, certains abus moraux. La plateforme _Mechanical Turk_ permet à tous et à toutes de récupérer des masses de documents en payant une autre personne pour le faire. Cette personne reçoit un salaire journalier de quatre à cinq dollars par jour [@semuels_online_nodate] pour des actions répétitives. En supposant qu'une journée de travail dure huit heures, que chaque document prend 30 secondes à télécharger et qu'il y a un nombre de documents suffisant pour au moins une journée, le téléchargement d'un document est payé entre 0.004$ et 0.005$. Avec de telles sommes, une nouvelle catégorie de précarité est ouverte puisqu'il n'atteint que 50% du salaire le plus bas d'Europe [@noauthor_monthly_nodate].

Les *turkers*, c'est-à-dire les employés de ces plateformes, dépendent de ce maigre revenu pour survivre. Ils ne sont pas toujours en mesure de quitter leur précarité et éprouvent donc une grande difficulté à se séparer de ces plateformes.

Soutenir de telles plateformes est perçu par l'opinion publique comme amoral, qui vont même jusqu'à parler "d'esclavage moderne" [@soyez_turc_nodate]. La plateforme d'Amazon n'est qu'un exemple parmi d'autres : de tels ateliers de misère similaires à _Mechanical Turk_ existent par dizaines. D'autres services, utiles au scraping, proposent la résolution de captcha par des humains ($0.001\$$ par captcha) dans des pays pour la plupart pauvres. La figure 1.3 contient les statistiques des pays dans lesquels résident les employés d'une société de résolution de captcha, pour la majorité en développement.

\cimg{figs/anticaptcha-stats.png}{scale=1.2}{Statistique des origines des travailleurs d'un service similaire à Mechanical Turk nommé Anti Captcha}{Source : tiré de \url{www.anti-captcha.com}, ref. URL05}

En utilisant un (+SAAS_a), on ignore s'il y a utilisation de plateformes telles que celles citées plus haut. Pour éviter ce genre de situation, l'utilisation de logiciels open source tels que les frameworks [@ben_10_2017] semblent non pas protéger contre ce genre de problèmes, mais d'avoir au moins la possibilité d'aller vérifier. Avoir un contrôle total du comportement de
l'application permet d'éviter l'utilisation non-informée d'une plateforme exploitant des êtres humains.

### Comparaison légale

En étudiant différents (+SAAS_a) et frameworks, aucun n'a semblé violer la loi à la connaissance de l'auteur. Certains (+SAAS_a) utilisent des techniques de détournement de blocages du fournisseur, tels que l'utilisation d'un proxy, d'un (+VPN_a) ou d'un pool d'adresses (+IP_a).

Certains se servent en outre de services de résolution de captcha, comme mentionné plus haut. Outre l'aspect de sous-payer des travailleurs dans des pays dont la plupart sont en difficulté, l'aspect légal de contourner une protection est tout à fait douteux. Une étude de l'université de San Diego [@motoyama_re_nodate] décrivait l'acte de contourner les captchas :_"it can be argued that CAPTCHAs are an access control mechanism and thus evading them exceeds the authorization granted by the site owner, in potential violation of the Computer Fraud and Abuse Act (and certainly of their terms of service)."_ (on peut soutenir que les CAPTCHAs sont un mécanisme de contrôle d'accès et que les éviter dépasse les autorisations données par le propriétaire du site, en violation potentielle avec la loi sur la sécurité des systèmes d'information et certainement de leurs conditions de service).
C'est toutefois un sujet à controverse. L'avocate Jennifer Granick indique, en parlant des captchas :_"Technologically and legally captchas can be thought of as nothing more than a speed bump as opposed to a barrier"_ ("D'un point de vue technologique et juridique, les captchas peuvent être considérés comme un simple ralentisseur plutôt qu'une barrière") [@zetter_is_2010].
Certains systèmes particulièrement perfectionnés sont à même de simuler un comportement humain [@zetter_is_2010].

Employer des techniques douteuses n'est pas puni par la loi mais peut être vu du public comme socialement inacceptable [@soyez_turc_nodate]; si une entreprise dépendant de sa renommée utilisait de telles techniques (et que cela se venait à s'apprendre), cela pourrait nuire à son image de marque.

La question juridique du scraping est complexe. C'est un sujet décrié par les grandes entreprises [@bode_court_2019], qui semble commencer à être réglementé [@igor_us_2018]. La règle d'or semble être de faire preuve de bon sens en ne causant aucun tort à l'entreprise fournissant des données[^8]. Pour pouvoir le vérifier, un contrôle total de l'application est nécessaire, ce qui est uniquement possible avec un framework.

[^8]: Ne pas faire preuve de fair-play en téléchargeant les données privées d'un site sans en demander la permission est un exemple d'une entreprise ne faisant pas usage du bon sens.

### Bilan intermédiaire

Au vu de ce qui a été mentionné, un logiciel à fabriquer, Inari, serait un logiciel possédant les fonctionnalités suivantes :

* Gratuit et open source : un tel logiciel serait alors extensible par la communauté utilisant ce logiciel et cela augmenterait la confiance en ce logiciel. En ce sens, il faudrait alors un framework.
* Facilement utilisable : un utilisateur quelconque devrait être en mesure de se servir d'un tel logiciel.
* Moral : un tel logiciel devrait mettre en place des mesures afin de pouvoir donner toutes les occasions à l'utilisateur de rester dans une utilisation la plus sûre possible du point de vue moral.

## Applications potentielles

Quelques applications d'un tel logiciel sont proposées ci-dessous. Cette liste est non exhaustive et un logiciel tel qu'Inari pourrait être utilisé à d'autres fins.

* **Téléchargement de documents** : L'utilisation la plus normale d'Inari rentre dans la capture et le téléchargement d'informations sous forme de documents. Cela consiste principalement à visiter un site, télécharger les documents et éventuellement traiter ces documents pour y récupérer une information en particulier.
* **Remplissage de documents** : A l'inverse de récupérer des informations, Inari pourrait très bien remplir des formulaires répétitifs, comme la prise de rendez-vous, des formulaires de contact, etc.
* **Traitement de documents** : Il est possible de se servir d'Inari ni pour la collecte de documents, ni pour le remplissage de formulaires, mais pour traiter rapidement des documents. Par exemple, récupérer une information particulière dans de nombreux documents similaires. Si par exemple $2000$ documents contenaient des informations au même endroit, il serait possible avec Inari d'itérer sur chaque document pour récupérer l'information en question.
* **Activités illicites** : Inari pourrait théoriquement, au vu de sa capacité à être distribué, être utilisé de manière malveillante pour empêcher d'accéder à un site. Il suffirait pour cela de lancer un grand nombre de connexions sur un site. On parle alors de (+DDoS_a). Cette utilisation n'est évidemment ni cautionnée par l'auteur de ce travail, ni recommandée du point de vue légal ou du sens commun. A titre de rappel, les (+DDoS_a) sont punis par la loi.

## Bilan

Le choix d'un logiciel de scraping n'est pas évident; il demande une réflexion certaine afin de ne pas reproduire les mêmes erreurs que d'autres. Peu importe la solution choisie, une vigilance accrue doit être apportée pour respecter les règles, quelles qu'elles soient. Les services de scraping étant de façon générale mal vus par les entreprises, ces dernières n'hésitent pas à saisir la justice pour faire arrêter le scraping [@rummler_facebook_nodate] [@schaal_ryanair_2018] de leur site.

Les frameworks ne sont pas parfaits : de complexité variable et difficilement utilisables de surcroît par les utilisateurs n'étant pas familiers avec la programmation ou l'informatique en général. Ils permettent en outre un contrôle accru de la récupération des données, ce qui permet
d'éviter les problèmes cités plus haut.

Est-il possible de créer un framework facilement utilisable par un utilisateur ? En utilisant un tel framework, un utilisateur quelconque serait en mesure de récupérer rapidement les informations d'un site. Le chapitre suivant étudie la conception d'un framework accessible à l'utilisateur.

\pagebreak 