# Méthodologie

Ce chapitre s'intéresse à l'implémentation du logiciel Inari.

## Vocabulaire du projet

Ce projet propose un certain nombres de termes afin de simplifier la compréhension de l'architecture du système.

* Workflow : Le terme workflow désigne une liste dans un ordre précis de tâches.
* Module : une action prédéterminée (cliquer quelque part, ouvrir un navigateur, effectuer une addition, etc.).
* Tâche : une tâche est une instance d'un module.
* Worker : Un système permettant de démarrer plusieurs workflows sur la même machine.

## Cahier des charges

Plusieurs critères permettent de garantir un fonctionnement optimal lors de la récupération de documents sur différents sites web.

* **Distributivité** : Les différents workflows doivent être distribués, c'est-à-dire répartis sur plusieurs machines différentes. De cette façon, il est possible d'accélérer le téléchargement de données et de répartir la charge sur plusieurs machines.
* **Parallélisme** : Pour accélérer le téléchargement de données, il est encore possible d'utiliser les différents coeurs du processeur de chaque machine accueillant Inari.
* **Simple d'utilisation** : Un utilisateur sans formation préalable doit être en mesure de se servir de ce logiciel.
* **Robustesse** : Le logiciel doit être capable de résister aux pannes. Par exemple, si un client Inari venait à s'arrêter, il devrait être possible de le remettre en état pour relancer les tâches que le client était en train de faire.
* **Modulaire** : Le logiciel doit être en mesure de fonctionner sur n'importe quel site, peu importe sa technologie. Par exemple, le système doit être compatible avec un site fonctionnant essentiellement avec javascript.

Chacun de ces critères est discuté en détail dans ce chapitre.

## Architecture du projet

Comme indiqué plus haut, il est nécessaire de posséder une architecture de ce projet qui est distribuée et résistante aux pannes. En effet, plutôt que de confier le travail de téléchargement des documents à une seule machine, Inari est capable de répartir la charge sur un nombre important de machines. De ce fait, Inari résiste aux pannes (à l'exception d'une panne simultanée de toutes les machines) et est capable de continuer à fonctionner même dans un environnement dégradé.

Un certain nombre de composants sont présents dans l'architecture d'Inari. L'architecture peut être décomposée en deux parties : les clients Inari, chargés de récupérer les documents, et le contrôleur, chargé de coordonner les clients.

\cimg{figs/architecture.png}{scale=.5}{Représentation de l'architecture Inari}{Source : créée par Théo Pirkl}

Inari fonctionne sur un modèle client-serveur. Le client se charge de télécharger des documents demandés par le serveur. Quant au serveur, dans ce cas le contrôleur, il se charge de plusieurs tâches :

* L'interaction entre l'utilisateur et les clients;
* La récupération des documents téléchargés par chaque client;
* La gestion des workflows, c'est-à-dire leur création comme leur exécution;
* La répartition des tâches entre les clients.

La figure 2.1 montre le schéma suivi par un workflow à son lancement. Pour commencer, le client se connecte au serveur web et sélectionne un workflow (flèches un et deux). Là, il peut sélectionner les documents qu'il souhaite récupérer. Ce choix fait, l'utilisateur lance le workflow. Cela enregistre dans la base de données les tâches à exécuter (flèche trois). Ces tâches sont ensuite envoyées au serveur Inari (flèche quatre), qui répartit de façon équitable les tâches à effectuer
aux différents clients (flèche cinq). Les documents sont alors téléchargés par les clients qui, à chaque tâche effectuée, informent le contrôleur du succès ou de l'échec de la tâche (flèche six). Enfin, le client peut, une fois tous les documents récupérés, télécharger une archive des clients contenant l'ensemble des documents téléchargés jusqu'ici (flèche sept).

### Communication

Deux systèmes de communication sont ici utilisés. Il était en premier lieu nécessaire d'utiliser un protocole particulièrement populaire pour les communications entre un utilisateur et Inari, de façon à ce que l'utilisateur sache déjà s'en servir. HTTP était alors un choix idéal, représentant $21\%$ du trafic mondial d'Internet [@sandvine_2018_2018]. Ainsi, il est possible pour l'utilisateur de gérer la récupération des documents en passant par un navigateur web.
L'autre protocole utilisé est ZeroMQ. ZeroMQ est utilisé ici pour échanger entre le contrôleur et chaque client. ZeroMQ a été choisi car il permet d'envoyer sans contraintes un nombre important de messages à la seconde et possède une grande versatilité[@dworak_middleware_2011]. ZeroMQ est donc utilisé en "interne", de façon à accélérer la diffusion de messages client-serveur (ou serveur-client), tandis que le protocole HTTP est utilisé pour toutes les actions liées à l'utilisateur. L'utilisation d'HTTP permet aussi la création d'une (+API_a) de type (+REST_a), ce qui permet d'automatiser le comportement d'Inari à l'aide d'un logiciel tiers.

### Génération et distribution des tâches

En fonction des documents à télécharger et du workflow, il est parfois nécessaire de lancer un très grand nombre de tâches de façon à récupérer une quantité importante de documents. Inari propose donc deux modes :

* Le mode _single_
* Le mode _bulk_.

Le mode _single_ permet, comme son nom l'indique, de lancer un workflow une fois de façon à récupérer un groupe de documents. Le mode _bulk_, quant à lui, permet de lancer une grande quantité de workflows en une seule fois. Cela peut s'avérer utile dans le cas où rentrer les informations nécessaires au lancement de chaque tâche prendrait trop de temps. La figure 2.2 donne un exemple d'application du mode _bulk_.

\cimg{figs/bulk.png}{scale=.35}{Exemple d'application du mode bulk}{Source : créé par Théo PIRKL}

Les informations entrées dans le mode bulk sont combinées entre elles. Par exemple, à la figure 2.2, une tâche est lancée avec le paramètres `month` à 9, `sid` à 6307.9099 et `year` à 2017; une autre est lancée avec les paramètres `month` à 10, `sid` à 6307.9099 et `year` à 2017. Toutes les combinaisons des paramètres rentrés sont alors convertis en workflow. Cette façon de faire permet de rentrer  rapidement un nombre important de workflow. Afin d'éviter de charger le contrôleur inutilement, c'est le client qui calcule toutes les combinaisons. Le contrôleur, quant à lui, reçoit les workflow générés et se contente de les répartir entre les clients Inari disponibles.

Inari envoie de façon intelligente les workflows sur le client le moins chargé. Il consulte pour cela sa base de données, et sélectionne le client possédant le moins de workflows. Cela permet de garantir un fonctionnement équitable de chaque client, et en cas de panne, de ne pas avoir à relancer une grande partie des workflows.
Dans le mode _single_, un workflow est automatiquement assigné à un client Inari. Dans le mode _bulk_, en revanche, l'ensemble des workflows reçus sont distribués à tous les clients. De cette façon, si un utilisateur génère 200 workflows à exécuter puis 2000, les tâches sont malgré tout réparties également entre chaque client. Il y aura donc deux clients avec 1100 workflows et non un avec 200 et l'autre avec 2000.

### Parallélisme

Le parallélisme permet, de manière générale, d'accélérer le traitement de tâches. Inari utilise deux types de parallélisme :

* Le multiprocessing;
* L'utilisation d'un pool de clients Inari.

Le multiprocessing permet de mettre à contribution chaque coeur du processeur sur lequel est exécuté le client Inari. Le multiprocessing permet d'isoler chaque traitement. Le multithreading n'était pas utilisable dans le cadre de ce projet en raison de la technologie utilisée pour télécharger les documents [@noauthor_multithreading_nodate].

L'utilisation d'un pool de clients Inari, quant à elle, permet d'ajouter une couche de parallélisme : on pourrait comparer son utilisation à un multiprocessing de haut niveau, où tous les ordinateurs formeraient ensemble un seul processeur. Utiliser plusieurs ordinateurs apporte aussi une redondance dans le traitement des documents. Dans le cas où un ordinateur hébergeant un client Inari venait à tomber en panne pendant un traitement, il serait possible relancer les tâches. On pourrait ensuite choisir de les relancer sur le même ordinateur une fois son fonctionnement restauré, ou sur un autre. L'utilisation des deux technologies de parallélisme permet non seulement de prévenir des pannes, mais aussi d'utiliser de façon optimale les ressources de chaque ordinateur accueillant un client Inari.
De cette façon, il est possible de traiter plusieurs workflows en même temps. Le workflow n'est donc pas accéléré mais la quantité de workflows exécutables en simultané est augmentée.
Par exemple, si huit machines sont considérées comme clients et que chacune possède huit coeurs, il est théoriquement possible de lancer $64$ workflows à la fois. Théoriquement, car les workflows peuvent consommer des ressources importantes et lancer trop de workflow à la fois peut surcharger l'ordinateur sur lequel est hébergé un des clients Inari. Il est donc important, en configurant les clients Inari, de ne pas allouer à ces derniers la totalité des ressources système à disposition.
Le chapitre _mesures_ montre l'accélération observée lors des mesures grâce au parallélisme.

### Interface graphique 

Pour permettre à l'utilisateur de se servir simplement d'Inari, une interface web est proposée à l'utilisateur. En effet, les interfaces web possèdent l'avantage d'être consultables depuis n'importe où, sur n'importe quel navigateur, et sans rien devoir installer.

L'interface web permet à l'utilisateur de _dessiner_ son workflow : un certain nombre de modules est proposé avec l'application, permettant à l'utilisateur de créer son workflow facilement et directement après l'installation de l'infrastructure Inari.

\cimg{figs/inari-frontend-ui.png}{scale=.35}{Interface web Inari}{Source : créé par Théo PIRKL}

Comme la figure 2.3 le montre, l'interface est composée de la façon suivante : à droite, l'interface permet de _dessiner_ son workflow en ajoutant des modules (les boîtes) et en les interconnectant (les flèches). Les boutons en bas à droite permettent la construction, la sauvegarde du workflow et l'ajout d'un module. Le menu à gauche permet de naviguer entre les différentes ressources accessibles à l'utilisateur, comme les workflows, les modules, les clients et les clicks. Chaque client Inari propose de télécharger l'intégralité des documents récupérés jusqu'ici. En outre, il est possible à l'utilisateur de gérer ses _clicks_. Les clicks représentent une ressource permettant à l'utilisateur de décrire à Inari ce qu'il faut sélectionner sur un site web en particulier. Ces clicks sont représentés sous la forme d'une formule XPath, l'équivalent d'un langage de programmation permettant de désigner un ou plusieurs éléments sur une page structurée comme du (+HTML_a). La figure 2.4 donne un exemple de clicks insérés dans l'application.

\cimg{figs/inari-frontend-xpath.png}{scale=.32}{Exemple de clicks}{Source : créé par Théo PIRKL}

Programmer une XPath n'est pas un prérequis prévu par le cahier des charges. Il suffit à l'utilisateur d'installer un module complémentaire Firefox, nommé _Inari Express_ permettant de les générer pour lui. Le module complémentaire consiste simplement en un viseur capable de sélectionner un élément dans la page, et de générer la XPath correspondant. Il est ensuite possible d'importer ce qu'a généré le module complémentaire dans l'interface web.

Un utilisateur se servirait de l'interface de la façon suivante : après s'être inscrit et s'être connecté, il aurait la possibilité de créer un workflow. Là, il pourra ajouter des modules, et les connecter entre eux. Une fois son workflow terminé, il aura la possibilité de le sauvegarder et de le générer. L'action de générer un workflow convertit le workflow du format visuel au format d'exécution. Ces deux formats sont discutés plus en détail dans la section _fonctionnement du workflow_.

Une fois son workflow généré, l'utilisateur a la possibilité de contrôler via la page _clients_ l'avancement de la récupération.

### Robustesse

Inari a été conçu de façon à pouvoir résister aux erreurs le mieux possible. La majorité des composants sont dépendants les uns des autres au moment où ils discutent entre eux, mais pas avant ou après la discussion.
Par exemple, le contrôleur Inari n'est pas essentiel au fonctionnement de l'application une fois que ce dernier a transmis aux clients Inari leurs tâches respectives. Un client Inari peut subir une panne : le contrôleur est en mesure de renvoyer les tâches d'un client Inari pour reprendre là où le client en était.
Enfin, l'interface de contrôle web est complètement détachée du reste du projet : même si elle tombe en panne, aucun autre composant ne sera impacté.
L'exécution des tâches est elle aussi prévue pour résister aux pannes : chaque client Inari signale chaque tâche comme terminée ou en échec. Si trop d'échecs surviennent sur un client, le contrôleur Inari ordonne automatiquement son arrêt temporaire. L'arrêt se caractérise comme une pause de cinq minutes, le temps au client Inari de terminer les tâches en cours. Cette pause de cinq minutes est particulièrement utile lors du ralentissement d'un site web : si le site sur lequel Inari tente de récupérer des documents commence à ralentir, cela s'observe sous forme d'une vague d'erreurs au niveau des tâches. Le contrôleur bloque alors les clients Inari, le temps que le site web retrouve ses performances.
Enfin, le parallélisme des tâches amène lui aussi une robustesse : chaque exécution de workflow est isolée des autres, ce qui permet d'éviter des effets de bords. Si un workflow devait s'arrêter en cours de route, il serait signalé au contrôleur, renvoyé au client et réexécuté dès que possible.

### Fonctionnement du workflow

Le workflow se présente à l'utilisateur sous la forme d'un graphe unidirectionnel, où chaque noeud correspond à un module (ou action). Chaque noeud possède un nombre prédéterminé d'entrées et de sorties. Le graphe est unidirectionnel afin d'indiquer un ordre d'exécution. La figure 2.5 donne un exemple de workflow.

\cimg{figs/inari-example-workflow.png}{scale=.28}{Exemple de workflow}{Source : créé par Théo PIRKL}

Une fois le graphe dessiné, il est nécessaire au système de convertir le format permettant de dessiner et d'afficher le workflow vers un format permettant de l'exécuter. Ce dernier n'est pas du code, car Inari ne possède pas de moteur syntaxique lui permettant d'autogénérer du code. A la place, un fichier contenant les modules à exécuter avec leurs arguments est généré. De cette façon, il suffit au client de démarrer le module indiqué par le fichier généré et d'indiquer les arguments nécessaires au fonctionnement du workflow.

\begin{figure}[t!]
    \begin{subfigure}{.48\linewidth}
        \includegraphics[width=\linewidth]{figs/format-1.png}
    \end{subfigure}\hspace*{\fill}
    \begin{subfigure}{.48\linewidth}
        \includegraphics[width=\linewidth]{figs/format-2.png}
    \end{subfigure}\medskip

    \caption{Comparaison du format d'affichage du workflow (gauche) et du format d'exécution Inari (droite)} \label{fig:1}
    \caption*{\textit{Source : réalisé par Théo Pirkl}}
\end{figure}
L'utilisation de deux formats est nécessaire, de façon à conserver les informations utiles à l'affichage client uniquement du côté client. Le fichier devient alors plus léger. A titre de comparaison, un fichier d'affichage pèse $49$Ko tandis que le format d'exécution n'en pèse que $24$. Les deux formats sont visibles sur la figure 2.6.

\pagebreak
Le format d'affichage se contente d'indiquer comment redessiner ce que l'utilisateur a créé, tandis que le format d'exécution permet véritablement de lancer un workflow créé par l'utilisateur. La conversion du premier format vers le second nécessite de déterminer quels sont les derniers modules à démarrer, de façon à pouvoir ensuite remonter le graphe jusqu'aux premiers composants. Cette action permet de déterminer l'ordre d'exécution des modules. Tout module possédant des entrées mais sans sortie est considéré comme module s'exécutant en dernier. Une fois le graphe remonté aux premiers modules à exécuter, Inari va lire les liens entre modules, de façon à indiquer que la valeur d'une entrée d'un des modules peut venir d'un module ayant déjà été exécuté.

\begin{figure}[t!]
    \begin{subfigure}{.48\linewidth}
        \includegraphics[width=\linewidth]{figs/example-dependency-1.png}
    \end{subfigure}\hspace*{\fill}
    \begin{subfigure}{.48\linewidth}
        \includegraphics[width=\linewidth]{figs/example-dependency-2.png}
    \end{subfigure}\medskip

    \caption{Exemple de dépendance entre modules}
    \caption*{\textit{Source : réalisé par Théo Pirkl}}
\end{figure}

La figure 2.7 montre un exemple de comment Inari indique à un module qu'un argument sera passé par un module précédent. En donnant l'ID unique du module ainsi que le nom de l'argument il est possible de récupérer la valeur nécessaire. Un tel système pourrait s'apparenter à un pointeur. Chaque tâche possède, comme indiqué plus haut, un certain nombre d'entrées et de sorties. Les sorties de chaque tâche sont conservées dans un tableau de façon à pouvoir être utilisées plus tard en cas de besoin.
Si un argument commence et finit par _$_, il est considéré comme un pointeur. Le pointeur est composé de deux parties, séparé par un point. Ci dessous, un exemple de pointeur :

`$3f7d4753-8a1b-4f5e-a108-71c9e3854690.output$`

La première partie du pointeur correspond ici à l'ID unique du module. C'est cette première information qui permet de savoir si le pointeur pointe sur une ancienne tâche valable ou non. La seconde partie du pointeur correspond à l'argument de sortie de l'ancienne tâche. Par exemple, si une tâche avait comme ID unique `foo` et comme argument de sortie `bar`, le pointeur serait `$foo.bar$`. La figure 2.8 montre comment le système détecte et récupère la valeur d'un pointeur.

\cimg{figs/pointer.png}{scale=.45}{Détection et lecture d'un pointeur}{Source : créé par Théo PIRKL}

## Catégories de scrapers

Les workflow sont répartis en plusieurs catégories, notamment les scrapers simples et ceux complexes. Un scraper correspond à un workflow. Cette section s'intéresse aux différentes catégories de scrapers.

## Catégorie une : Scrapers simples

Les scrapers simples correspondent aux workflows où la récupération de documents se limite à consulter une page web et où le traitement (clicks, remplissages de formulaires) est minimal ou inexistant. Cela correspond à tous les sites ne dépendant pas de JavaScript pour naviguer.

### Application pratique : FAO Genève

La FAO Genève est un site de la République et Canton de Genève, lequel publie des documents, notamment des actes officiels, et des avis administratifs ainsi que judiciaires. Les documents sont disponibles sous forme de liste, où chaque document peut être téléchargé en cliquant sur un bouton. Le seul traitement nécessaire est de passer de page en page de façon à récupérer d'anciens documents.

Pour récupérer les documents, la marche à suivre est la suivante : lancer un navigateur, puis le faire naviguer sur le site web de la FAO Genève (\url{https://fao.ge.ch}). Là, un captcha est à résoudre, mais qui peut être évité en remplaçant dans l'URL `captcha` par `login`. C'est ce qui est alors fait. Une fois le captcha détourné, on accède à la liste des fichiers récents. Il faut alors exécuter une XPath pour retrouver l'URL de téléchargement de chaque document : on simule ensuite ces clics pour récupérer les documents en une seule fois.
Le traitement principal consiste donc à aller sur le site, contourner le captcha et détecter les boutons de téléchargement, ce que fait le workflow Inari.
Le traitement ci-dessus correspond à une page. Pour télécharger 200 pages, il suffit d'ajouter une étape où une fois que le captcha est résolu, on demande au navigateur d'aller à la page correspondante via le paramètre `page` dans l'URL. Par exemple, la page 40 correspond à l'URL \url{https://fao.ge.ch?page=39}. 

## Catégorie deux : Scrapers complexes

Les scrapers complexes correspondent aux workflows où la récupération de documents demande un traitement conséquent sur la page comme remplir un ou plusieurs formulaires. De manière générale, cela correspond soit aux sites possédant des formulaires à remplir, soit aux sites où la navigation est assurée par Javascript. Dès lors qu'il est nécessaire de faire du prétraitement pour atteindre chaque document, le workflow est considéré comme complexe.

### Application pratique : Swiss-Impex

Swiss-Impex est un site où les statistiques d'import-export suisses sont publiées. Le site fonctionne avec un framework JavaScript et nécessite, pour chaque document, de remplir de nombreux formulaires. Par conséquent, il s'agit d'un workflow complexe. La figure 2.9 montre quelques écrans à remplir pour obtenir un document.

\begin{figure}[t!]
    \begin{subfigure}{.48\linewidth}
        \includegraphics[width=\linewidth]{figs/swiss-impex-1.png}
    \end{subfigure}\hspace*{\fill}
    \begin{subfigure}{.48\linewidth}
        \includegraphics[width=\linewidth]{figs/swiss-impex-2.png}
    \end{subfigure}\medskip

    \begin{subfigure}{.48\linewidth}
        \centering
        \includegraphics[width=\linewidth]{figs/swiss-impex-3.png}
    \end{subfigure}\hspace*{\fill}
    \begin{subfigure}{.48\linewidth}
        \includegraphics[width=\linewidth]{figs/swiss-impex-4.png}
    \end{subfigure}\medskip

    \caption{Les nombreux écrans de Swiss-Impex nécessaires à remplir pour obtenir un document}
    \caption*{\textit{Source : \url{www.gate.ezv.admin.ch}, URL06}}
\end{figure}

Il faut commencer par remplir la date du document que l'on souhaite obtenir. Il faut ensuite choisir parmi les 9000 catégories de document celle que l'on souhaite obtenir, puis sélectionner les pays d'import ou d'export de la marchandise sélectionnée. Enfin, il est possible d'inclure ou non les matières précieuses comme l'or, avant de pouvoir exécuter la requête. Une fois le document généré, il faut changer quelques réglages esthétiques avant de pouvoir exporter le résultat sous forme de XLSX. Cependant, ce format est propriétaire et peu pratique. Il est donc nécessaire de l'exporter en (+CSV_a). C'est encore une action qui est nécessaire d'être effectuée avant de pouvoir passer au prochain document.

Dans ce cas, les actions sont nombreuses, et chaque action est synchrone. De plus, chaque clic est envoyé au serveur Swiss-Impex. A chaque
envoi, les formulaires sont verrouillés et il est donc nécessaire d'attendre à chaque clic la réponse du serveur. Il n'est pas possible de récupérer plus qu'un mois de valeurs à la fois, c'est-à-dire un document relatant d'une marchandise pour un mois pour tous les pays en
import ou export. En effet, le site Swiss-Impex limite le nombre de cellules que l'on peut demander à la fois.

## Catégorie 1.5 : Scrapers hybrides

Certains sites ne rentrent pas entièrement dans l'une des deux catégories : un traitement spécial est nécessaire. Il peut être au niveau d'une limitation de connexion, d'adresse IP, ou une autre limitation à respecter.

### Application pratique : Registre du Commerce

Le Registre du Commerce est un exemple de site nécessitant une attention particulière au nombre de connexions simultanées. Ce site recense les extraits d'entreprises de tout le canton de Genève.
Les documents sur le site sont générés à la demande : afin d'éviter de surcharger le service, il est donc nécessaire de limiter le nombre de documents générés à la fois en limitant le nombre de connexions simultanées. De plus, le site ne propose la génération que d'un document à la fois. Il n'est pas possible de générer un document contenant tous les résultats.

## Bilan

L'infrastructure d'Inari est faite de plusieurs composants relativement complexes.
Toutefois, du point de vue de l'utilisateur, c'est une solution facile à prendre en main, car aucune programmation n'est nécessaire, aucune installation n'est requise puisqu'il suffit d'un navigateur web et aucune notion en informatique n'est nécessaire.
Cette infrastructure permet aussi de mitiger les pannes, donc d'assurer une continuité des opérations même en cas de problème sur l'un des composants.
Une fois que l'infrastructure est installée, il ne reste plus qu'à effectuer des mesures de performance afin de déterminer la résistance de l'infrastructure face aux montées en charge.

\pagebreak