# Performances d'Inari

Ce chapitre traite des performances d'Inari. Ces dernières donnent non seulement un aperçu de la vitesse de traitement des documents, mais permettent aussi de calculer un pourcentage de gain de performance par rapport à un humain. Grâce à ces deux mesures, il est non seulement possible d'estimer le temps nécessaire au téléchargement d'un bloc de documents, mais également le temps gagné par rapport à une opération manuelle (c'est-à-dire opérée par un humain).

Plusieurs sources de données sont analysées : chaque source de documents propose une façon différente de récupérer le document. Une première série de mesures seront faites tout d'abord sur la (+FAO_a) du canton de Genève. Une seconde série de mesures sera effectuée sur le service Swiss-Impex. Une dernière sera lancée sur le (+RC_a) du canton de Genève.

Le but de ce chapitre est de répondre à deux questions :

* Est-ce que l'automatisation de la récupération des documents est plus rapide que le téléchargement des documents un par un ?
* Est-ce que la distribution des tâches de téléchargement accélère le traitement des tâches ?

L'intégralité des mesures effectuées sont disponibles [sur le git associé au projet](https://githepia.hesge.ch/theo.pirkl/bachelor-scrapper-058/-/tree/master/projet/agent/resources/measures/2).

## FAO Genève

La (+FAO_a) Genève publie régulièrement durant la semaine une multitude de documents allant de l'avis de police à l'annonce de démolition. Toute personne peut obtenir par ce biais des informations la concernant. A la différence de courriers envoyés aux intéressés (annonce par courrier), la (+FAO_a) fonctionne par publication : il est donc théoriquement nécessaire à chacun de rester informé en permanence des publications intervenant sur ce site.

### Théorie

La (+FAO_a) ne propose pas de système d'abonnement et ne propose pas non plus de télécharger les documents en un bloc. Il est donc  nécessaire à tout usager du site de télécharger les documents un par un.

Inari propose un système permettant le téléchargement de documents en bloc, c'est-à-dire de télécharger les documents en une seule fois plutôt qu'un par un. Les mesures sont effectuées sur ce téléchargement en bloc.

### Mesures

Les expériences ont été effectuées sur plusieurs environnements :

* Une machine, avec deux travailleurs puis huit;
* Deux machines, avec deux travailleurs puis huit;
* Dix machines, avec deux travailleurs puis huit.

Chaque expérience a été effectuée trois fois. L'objectif d'augmenter le nombre de machines est d'observer si la vitesse de téléchargement des documents est linéaire ou non.

### Résultats

\begin{samepage}

Chaque expérience avait le même objectif : télécharger les 1000 mêmes documents du site. Ces dernières correspondent donc à une exécution complète du logiciel et chacune d'entre elles est indépendante des autres. Les clients Inari ont été redémarrés après chaque expérience pour garantir une fiabilité maximale des résultats.
Puisqu'il y a six expériences, six graphiques sont présentés. Chaque graphique comporte trois lignes, pour les trois mesures de chaque expérience. On télécharge $1000$ documents, répartis dix par dix sur des pages; on télécharge donc 100 pages. L'ordonnée des mesures ira donc de zéro à 100 pages téléchargées.
Le terme \textit{document} sera privilégié à celui de \textit{page}. On préfèrera le terme \textit{expérience} à celui de graphique : la figure 3.1 contient les six graphiques de chaque expérience. Une lettre est indiquée en légende sous chaque graphique et elle indique la lettre de l'expérience.

\begin{figure}[t!]
    \begin{subfigure}{0.48\textwidth}
        \includegraphics[width=\linewidth]{figs/measures/fao/1m2c.pdf}
        \caption{Une machine, deux coeurs} \label{fig:a}
    \end{subfigure}\hspace*{\fill}
    \begin{subfigure}{0.48\textwidth}
        \includegraphics[width=\linewidth]{figs/measures/fao/1m8c.pdf}
        \caption{Une machine, huit coeurs} \label{fig:b}
    \end{subfigure}

    \medskip
    \begin{subfigure}{0.48\textwidth}
        \includegraphics[width=\linewidth]{figs/measures/fao/2m2c.pdf}
        \caption{Deux machines, deux coeurs} \label{fig:c}
    \end{subfigure}\hspace*{\fill}
    \begin{subfigure}{0.48\textwidth}
        \includegraphics[width=\linewidth]{figs/measures/fao/2m8c.pdf}
        \caption{Deux machines, huit coeurs} \label{fig:d}
    \end{subfigure}

    \medskip
    \begin{subfigure}{0.48\textwidth}
        \includegraphics[width=\linewidth]{figs/measures/fao/10m2c.pdf}
        \caption{Dix machines, deux coeurs} \label{fig:e}
    \end{subfigure}\hspace*{\fill}
    \begin{subfigure}{0.48\textwidth}
        \includegraphics[width=\linewidth]{figs/measures/fao/10m8c.pdf}
        \caption{Dix machines, huit coeurs} \label{fig:f}
    \end{subfigure}

    \caption{Performances du workflow FAO Genève} \label{fig:1}
    \caption*{\textit{Source : réalisé par Théo Pirkl}}
\end{figure}

\end{samepage}

L'environnement qui a permis de réaliser l'expérience A est celui le plus comparable à un être humain au vu du nombre de travailleurs. En effet, cet expérience comporte deux automates, là où tous les autres en comportent plus.
Il a été établi empiriquement qu'un être humain récupère $50$ documents par minute. Il faudrait alors $200$ minutes (soit trois heures et 20 minutes) pour récupérer $1000$ documents. En départageant la tâche en deux (en demandant à deux humains de récupérer chacun une moitié des $1000$ documents), il faudrait alors théoriquement[^3] $100$ minutes, c'est-à-dire une heure 40.

Pour calculer le gain de performance, on prend le temps nécessaire à un humain et on le divise par le temps moyen observé sur chaque mesure :
$Taux_{performance} = Temps_{humain}/Temps_{machine}$. Pour obtenir le pourcentage de gain de performance, il faut utiliser la formule suivante :
$Pourcentage_{performance} = Taux_{performance} \cdot 100 - 100\%$. Soustraire 100% du pourcentage obtenu permet de voir le gain de temps gagné par la machine sur l'humain.

Inari télécharge ici deux documents en même temps et prend dans le pire des cas $500$ secondes, c'est-à-dire huit minutes et 20 secondes.
C'est une performance $12$ fois supérieure à celle d'un être humain, soit $1100\%$ de gain d'efficacité. Dans le meilleur des cas (ici, la mesure un), on prend $400$ secondes à télécharger $1000$ documents (soit 2.5 documents par seconde, ou 1.25 documents par seconde par coeur), soit une performance de 15 fois la vitesse d'un être humain.

En se basant sur les résultats présentés dans l'expérience A, l'expérience B devrait idéalement prendre quatre fois moins de temps, puisque quatre fois plus de coeurs sont mobilisés. Si on considère que la mesure du temps moyen de l'expérience A est de $480$ secondes, la B montre un facteur de gain de temps de quatre. On prend donc $120$ secondes à télécharger $1000$ documents (soit 8.3 documents par seconde ou 1.03 documents par seconde par coeur). Toujours dans l'expérience B, on observe une performance accrue d'un facteur 12.5 par rapport à un humain, soit un pourcentage de $1150\%$.

L'expérience C révèle l'avantage de la distribution de calcul. Théoriquement, on s'attend à diviser par deux le temps de traitement par rapport à l'expérience A, puisque l'on passe de deux coeurs à quatre. Le temps nécessaire au téléchargement devrait alors théoriquement être environ égal à $240$ secondes. Toutefois, l'expérience C montre un temps de téléchargement constant plus bas, à $175$ secondes, soit une vitesse à 5.7 documents par seconde (ou 1.42 documents par seconde par coeur). Cela peut s'expliquer de plusieurs façons : tout d'abord, chaque machine a un nombre fini de ressources : il est donc plus intéressant de répartir la charge sur plusieurs machines que sur plusieurs coeurs de la même machine. De cette façon, leur charges respectives sont réduites. Une autre explication serait liée à la façon dont les serveurs web fonctionnent. Si plusieurs ordinateurs se connectent, le serveur peut éventuellement identifier cela comme du trafic légitime.
Dans tous les cas, l'expérience C montre un gain de performance de facteur 17.14, soit un pourcentage de $1614\%$.

L'expérience D semble marquer la fin de l'observation jusqu'ici faite du gain linéaire entre le nombre de coeurs totaux et le temps nécessaire au téléchargement des documents. En effet, en reprenant l'expérience B, il faut $120$ secondes à huit coeurs pour télécharger $1000$ documents, soit 8.3 documents par seconde ou 0.51 documents par seconde par coeur. L'expérience D prendrait alors idéalement deux fois moins de temps que l'expérience B, c'est-à-dire $60$ secondes. En pratique, l'expérience D montre que l'on en prend plutôt $100$ secondes. Ce temps semble trop éloigné de la valeur théorique de 60 secondes. Le modèle jusqu'ici présumant d'une diminution linéaire du temps par rapport au nombre de coeurs est donc invalide. Un autre modèle est estimé à partir des mesures présentes à la fin de cette section.
L'expérience D montre un gain de performance de facteur 7.5, soit un pourcentage de $650\%$. Ces valeurs sont plus basses que les précédentes. Cette diminution devrait aussi apparaître dans les mesures suivantes.

L'expérience E semble confirmer que le modèle linéaire n'est pas adéquat. En effet, on observe un facteur de gain de performance de $8.57$, soit $757\%$ de gain. La théorie du modèle linéaire indique que dans la configuration de l'expérience E (c'est-à-dire 10 machines, deux coeurs) on arriverait à un temps de traitement de 35 secondes. 75 secondes sont nécessaires pour télécharger les 1000 documents, soit une vitesse de 13 documents par seconde, ou 0.65 documents par seconde par coeur.

Enfin, l'expérience F montre un gain de performance de facteur $3$, soit un pourcentage de $200\%$. Il a fallu 55 secondes pour télécharger les 1000 documents, soit une vitesse de 18 documents par seconde, ou 0.225 documents par seconde par coeur.

[^3]: Sans erreurs, sans hésitation, et sans arrêt.

Le contenu des documents téléchargés sort du cadre de ce travail. L'analyse des taux de performance méritent toutefois d'être analysés de façon plus profonde.

\cimg{figs/measures/fao/perf.pdf}{scale=0.5}{Graphique du score de performance en fonction du nombre de coeurs}{Source : réalisé par Théo Pirkl}

Comme le montre la figure 3.2, les meilleures performances semblent se situer du côté gauche du graphique. La meilleure performance observée correspond à l'expérience B (deux machines deux coeurs) et la plus mauvaise performance est observée sur l'expérience F.
Cela peut s'expliquer de plusieurs façons : l'expérience F est trop puissante et requiert un trop grand nombre de téléchargement, occasionnant une attaque (+DDoS_a) qui ralentit l'intégralité du processus. L'expérience B, quant à elle, pourrait correspondre à un ratio idéal pour le site web de la FAO permettant d'atteindre de telles performances.
Il est toutefois difficile de tirer de vraies conclusions de ce graphique : en effet, la quasi totalité des mesures se situe sur la moitié gauche du graphique. Une seule mesure a été effectuée sur un grand nombre de coeurs. Pour tirer une conclusion, il serait nécessaire de lancer un téléchargement avec 30, 40, 50 puis 60 coeurs. On peut toutefois sans ces mesures déduire que le nombre de coeurs totaux n'est pas étroitement lié au taux de performance, sans quoi le taux de performance augmenterait plus ou moins en fonction du nombre de coeurs. On observe ici plutôt l'inverse. Il serait utile de tester le téléchargement de documents avec plusieurs configurations différentes, afin de trouver le réglage optimal pour le site en question. Ce réglage est lié au site et à ces capacités de montée en charge.

### Bilan

Deux questions ont été posées dans l'introduction de ce chapitre. La première question était de déterminer si l'automatisation de la récupération des documents était plus rapide que le téléchargement des documents un par un. Au vu des mesures, on peut conclure que oui. Dans le pire des cas, c'est-à-dire les cas les plus lents, Inari est capable d'effectuer les actions trois fois plus vite qu'un être humain.

La seconde question, quant à elle, concernait la distribution des tâches de téléchargement : est-ce que cela accélère le traitement des tâches ? Il est difficile de répondre catégoriquement sur la base des résultats obtenus. Il faut trouver les réglages idéaux afin de pouvoir véritablement accélérer le traitement des documents. Dans l'ensemble des mesures prises, les paramètres donnant les pires taux de performances restent bien plus rapides qu'un être humain. Toutefois, on peut passer d'un taux de performance de $200\%$ à $1614\%$ en optimisant les réglages. C'est donc un processus pouvant se montrer indispensable. A noter que la variation du taux de performance est quelque chose qui est non maîtrisable, puisqu'il dépend principalement du serveur sur lequel les documents sont téléchargés.

## Swiss-Impex

Swiss-Impex est une base de données proposant l'intégralité des statistiques d'import-export de toutes les marchandises en Suisse. Les statistiques remontent jusqu'à 1988 et vont jusqu'à 2020.

### Théorie

Swiss-Impex demande plus de travail que la FAO Genève pour télécharger un document. La FAO Genève propose une page où l'on peut directement récupérer le document, tandis que Swiss-Impex demande de remplir cinq pages de formulaires afin de pouvoir fournir le document adéquat.

Il a été établi empiriquement qu'un être humain prend une minute pour télécharger un document. L'utilité d'un automate pour récupérer ne serait-ce qu'une catégorie de documents prend ici tout son sens. En effet, les statistiques sont détaillées mois par mois, et les documents remontent dans la majorité des cas jusqu'en 1988. Dans ces cas, on obtient un total de $372$ documents par catégorie ($mois \cdot (annee - 1) = 12 \cdot 31 = 372$).
Un être humain prendrait donc six heures et $12$ minutes pour télécharger une catégorie complète. Il y a environ $9000$ catégories sur le site de Swiss-Impex, soit un total de $3348000$ documents. Il semble difficile même pour un automate de télécharger l'intégralité du site. A une vitesse de 100 documents par minute et en ignorant certaines contraintes, un automate prendrait environ un mois pour télécharger l'intégralité du site.
Il est donc plus intéressant de s'intéresser à télécharger une seule catégorie.

L'expérience se concentre sur une catégorie en particulier, la catégorie *2709.0090*, marchandise correspondant selon le site de l'administration douanière aux *Huiles brutes de pétrole ou de minéraux bitumineux destinées à être utilisées comme carburant*.
Les données de cette catégorie ont été téléchargées sur dix machines, ayant chacune deux, six puis huit coeurs. Chacune de ces mesures a été effectuée trois fois.

Les données de plusieurs autres catégories ont ensuite été téléchargées pour observer le téléchargement sur le long terme. Ces catégories sont les suivantes :

* 6307.9099 (masques de protection en tissu)
* 9301.1000, 9301.2000, 9301.9000, 9302.0000, 9303.1000, 9303.2000, 9303.3010, 9303.3090, 9303.9000 (armes à feu)
* 2709.0090 (carburant)

Il y a $11$ catégories, ce qui fait $4092$ documents à télécharger. La mesure a été effectuée elle aussi à trois reprises.

### Mesures

Les mesures concernant seulement le pétrole sont les premières à être présentées. Les mesures concernant les masques en tissu, les armes à feu et le carburant sont analysés plus bas.

#### Analyses des imports-exports de matières pétrolières

\begin{samepage}

Comme le montre la figure 3.3, trois graphiques sont obtenus (un graphique par expérience) : chaque graphique représente le nombre de documents téléchargés en fonction du temps.

\begin{figure}[t!]
    \begin{subfigure}{0.48\textwidth}
        \includegraphics[width=\linewidth]{figs/measures/swiss-impex/10m2c.pdf}
        \caption{Dix machines, deux coeurs} \label{fig:a}
    \end{subfigure}\hspace*{\fill}
    \begin{subfigure}{0.48\textwidth}
        \includegraphics[width=\linewidth]{figs/measures/swiss-impex/10m6c.pdf}
        \caption{Dix machines, six coeurs} \label{fig:b}
    \end{subfigure}

    \medskip
    \begin{subfigure}{\textwidth}
        \centering
        \includegraphics[width=.48\linewidth]{figs/measures/swiss-impex/10m8c.pdf}
        \caption{Dix machines, huit coeurs} \label{fig:c}
    \end{subfigure}\hspace*{\fill}

    \caption{Performances du workflow Swiss-Impex} \label{fig:1}
    \caption*{\textit{Source : réalisé par Théo Pirkl}}
\end{figure}

\end{samepage}

Un premier phénomène peut s'observer : tous les agents (c'est-à-dire, chaque machine, indépendamment de son nombre de coeurs) ne réagissent pas tous de la même façon. C'est particulièrement visible dans l'expérience C : certains agents réagissent immédiatement à l'ordre tandis que d'autres prennent plus longtemps à agir. Ce phénomène est lié à l'aléatoire du réseau : le message est probablement arrivé plus tard sur certaines machines. L'élément réseau rend impossible de dire avec certitude de quoi il s'agit.
Si un être humain a besoin de six heures et $12$ minutes pour télécharger toute une catégorie, cela prendrait théoriquement 18 minutes à vingt humains.
Dans le cas d'Inari, les mesures indiquent que 20 automates simultanés (10m2c[^4]) prennent huit minutes. Le taux de performance gagné comparé à un être humain est de $2.25$, soit $125\%$, soit $0.775$ documents par seconde ou 0.03 documents par seconde par coeur.

Dans le cas de 60 coeurs (10m6c), 60 humains prendraient 6.2 minutes. Inari prend trois minutes et 20 secondes. Le taux de performance gagné est alors de $1.86$, soit de $86\%$, soit un document par seconde ou $0.016$ document par seconde par coeur.

Enfin, dans le cas de 80 coeurs, (10m8c), 80 humains prendraient 4.65 minutes. Inari prend deux minutes 40. Le taux de performance gagné est de $1.55$, soit un pourcentage de gain de $55\%$, soit $1.3$ document par seconde ou $0.016$ document par seconde par coeur.

On remarque que les pourcentages diminuent. Comme montré plus tôt, l'augmentation du volume de documents à récupérer de façon simultanée ralentit non seulement les machines mais aussi le site web. En l'occurence, Swiss-Impex est un site particulièrement sensible pouvant facilement cesser de fonctionner sous la charge. Il est donc, dans le cas de ce site, indispensable de trouver les paramètres corrects permettant de récupérer les documents sans que le site tombe en panne.

La prochaine mesure étant plus longue, l'analyse des imports-exports de matières pétrolières, des armes et des masques en tissu a été effectuée dans la configuration 10m2c, c'est-à-dire dix machines et deux coeurs de façon à garantir que Swiss-Impex ne tombe pas en panne.

[^4]: 10m2c correspond à 10 **M**achines, deux **C**oeurs.

#### Analyse des imports-exports de matières pétrolières, des armes et des masques en tissu

On obtient ici un graphique. Chaque ligne représente le nombre de documents téléchargés en fonction du temps.
\cimg{figs/measures/swiss-impex/special.pdf}{scale=.75}{Graphique du téléchargement de Swiss-Impex en configuration multicatégorie}{Source: créé par Théo Pirkl}

Pour commencer, il est intéressant de voir que le logiciel, même sur plusieurs milliers de documents, conserve une vitesse de téléchargement relativement constante, ce qu'on peut observer grâce à la quasi-droite qu'affiche le graphique. On peut apercevoir sur la mesure deux un ralentissement prononcé de la vitesse de téléchargement, qui peut être expliquée par une lenteur du service ou une lenteur des postes téléchargeant les documents.

Il faut, selon les mesures effectuées, $9800$ secondes pour télécharger $4092$ documents, soit 0.41 documents par seconde, ou 0.002 documents par seconde par coeur. On peut calculer un gain de performance de $1.25$ fois la vitesse d'un être humain, soit un pourcentage de gain absolu de $25\%$. Pour comparer ces mesures à celles de téléchargements plus courts, on remarque que le gain de performance sur des quantités de documents plus grandes semblent influencer le gain de performance final : dans le cas de quantités plus petites, le gain de performance était de $2.25$ la vitesse d'un être humain. Il est toutefois impossible d'affirmer qu'il existe une corrélation entre la quantité de documents à télécharger et le gain de performance : de nombreux facteurs externes tels que la vitesse du réseau, l'état du service et l'état des machines effectuant le téléchargement viennent influencer la mesure de façon quasi aléatoire.

### Bilan

La récupération des documents sur Swiss-Impex est, au vu des mesures, plus longue que sur la FAO Genève. Toutefois, Inari semble tout à fait apte à supporter la montée en charge. Pour reprendre les questions posées en début de chapitre :

* Est-ce que l'automatisation de la récupération des documents est plus rapide que le téléchargement des documents un par un ? Bien que les résultats ne soient pas aussi rapides que ceux du site précédent, on arrive malgré tout à des résultats plus rapides qu'un être humain. La réponse est donc oui. Malgré les statistiques hétérogènes, Inari apporte une fiabilité qu'un être humain ne peut apporter sur le long terme. Même quand le pourcentage de performance gagné semble faible par rapport à d'autres pourcentages, Inari continue à traiter ces documents, même suite à une erreur, sans arrêt et est capable de travailler 24 heures sur 24, sept jours sur sept, en reprenant les tâches échouées. Un humain ne peut apporter ces performances et serait impacté dans son travail s'il devait faire le travail d'Inari.
* Est-ce que la distribution des tâches de téléchargement accélère le traitement des tâches ? Oui, mais pas linéairement. Autrement dit, on observe par exemple qu'à 80 workers simultanés, la fréquence de téléchargement se rapproche de celle d'un humain.

## Registre du Commerce (RC)

Le site du Registre du Commerce de Genève recense toute les entreprises enregistrées dans ce canton. Le site permet de consulter l'extrait de chaque entreprise, lequel mentionne notamment le représentant, le siège social et le capital.

Ces informations peuvent s'avérer particulièrement utiles pour des sites comme les sites de gestion du risque, de renseignements d'entreprise, d'analyse de solvabilité, etc.

### Théorie

Le site du Registre du Commerce rappelle la plateforme FAO Genève : aucun formulaire n'est à remplir, et il suffit de cliquer sur un lien pour obtenir un document. Toutefois, la difficulté se situe dans le fait que le Registre du Commerce bloque les requêtes d'extrait répétitives. Il s'agit donc de télécharger les documents du Registre du Commerce sans se faire bloquer.
Pour tenter d'éviter le bloquage, les documents sont volontairement téléchargés moins vite, afin d'étaler la période de télécharger et donc d'éviter le bloquage.
Un autre problème réside dans l'application elle-même : si trop de demandes sont faites à la fois, l'application générant les extraits du commerce cesse de fonctionner, faute de capacité.

Il est donc difficile de télécharger les $48000$ documents sans déclencher un (+DDoS_a). Lors de la première mesure, consistant à télécharger les dix premières pages, le site a immédiatement montré des signes de lenteur caractéristiques d'un (+DDoS_a). La mesure a alors été immédiatement arrêtée.

Le site du Registre du Commerce est donc un site non téléchargeable, à cause du risque de rendre le site inopérant. Cette application montre les effets néfastes qu'un logiciel tel qu'Inari peut avoir sur un service. Il est donc particulièrement important pour l'utilisateur décidant de télécharger un site de faire attention à la vitesse de traitement des documents. Après examination, rien ne semble permettre une génération des documents de groupe, c'est-à-dire une génération de nombreux documents d'un coup plutôt que de les demander un par un. Or, lancer un trop grand nombre de téléchargement peut amener à un arrêt du site de plusieurs heures.
A noter que le droit suisse prévoit une peine de privation de liberté d'un an ou plus ou d'une peine pécuniaire dans ce genre de cas.

Au vu de ces constats, il est impossible de répondre aux questions posées en début de chapitre pour ce site.

## Bilan global des résultats

Pour répondre aux questions initialement posées, la majorité des sites analysés permettent non seulement d'aller plus vite qu'un être humain, mais la parallélisation permet d'accélérer davantage la vitesse de téléchargement. Au-delà de  répondre aux deux questions posées dans l'introduction de ce chapitre, il est possible de tirer certaines conclusions supplémentaires.

Tout d'abord, il est montré que le modèle prétendant que la vitesse de téléchargement est linéaire aux nombre de workers est erroné. La vitesse atteint un seuil maximum de vitesse, variable à chaque site web (et pouvant varier sur le même site web en fonction de nombreux facteurs comme la charge du site lors du téléchargement, le nombre de serveurs disponibles, la bande passante des workers, etc.). De façon générale, le modèle semble suivre la forme d'une parabole convexe, où le sommet correspond au nombre idéal de coeurs pour le site sur lequel on télécharge les documents.

Il a aussi été montré que dans certains cas (comme celui du Registre du Commerce), il n'était pas possible de télécharger les documents sans causer des problèmes au fournisseur du site. Le risque de mise en panne était trop grand.

Enfin, l'infrastructure Inari a pu résister à la montée en charge sans ralentissement véritable. Les ralentissements sur les récupérations à long terme montraient une lenteur de la part des sites hébergeant les données, mais pas de l'infrastructure Inari. Cette infrastructure semble donc résister aux montées en charge.

\pagebreak
