# Annexes {-}

\newappendix{Installation d'un client Inari}

Pour installer le client Inari, certains logiciels sont prérequis. Il est recommandé d'utiliser Ubuntu ou Debian. La liste des logiciels à installer est la suivante :

* Python 3.7 (la 3.8 est recommandée, mais les commandes ci-dessous utiliseront 3.7)
* Git
* Pip pour Python 3.7 (par défaut, pip est installé avec python sur les distributions populaires comme Debian ou Ubuntu)
* screen
* gcc

La première étape consiste à télécharger le projet.

Une fois le projet cloné, il faut lancer les commandes suivantes dans le dossier cloné :
```
cd projet/agent
python3.7 -m pip install -r requirements.txt
```

Le client est installé. Il faut maintenant créer un fichier de configuration dans le même dossier (`projet/agent`). Il suffit pour cela de faire la commande suivante : `cp example.env .env`. Il faut ensuite changer les réglages du fichier `.env` conformément à l'annexe _Fichier de configuration Inari_.

Il ensuite possible de démarrer le client Inari en lançant la commande suivante depuis le dossier `projet/agent` : `python3.7 src/start.py --remote`. `--remote` permet d'autoriser les connexions depuis l'extérieur.

Si Inari est lancé par SSH, il est recommandé de lancer Inari via `screen`. Pour cela, il suffit de taper la commande `screen`, puis de suivre les instructions du paragraphe précédent.


\pagebreak
\newappendix{Installation du contrôleur Inari}

Pour installer le contrôleur Inari, certains logiciels sont prérequis. Il est recommandé d'utiliser Ubuntu ou Debian. La liste des logiciels a installer est la suivante :

* Python 3.7 (la 3.8 est recommandée, mais les commandes ci-dessous utiliseront 3.7)
* Git
* Pip pour Python 3.7 (par défaut, pip est installé avec python sur les distributions populaires comme Debian ou Ubuntu)
* screen
* gcc
* PHP 7.3
* php-fpm
* php-yaml
* php7.3-mysql
* composer
* npm

Il faut lancer un serveur web pour accueillir la partie frontend du contrôleur. Il faut pour cela configurer Nginx sur le serveur. Un tutoriel présenté ici permet de configurer proprement Nginx : \url{https://www.digitalocean.com/community/tutorials/how-to-install-and-configure-laravel-with-nginx-on-ubuntu-20-04}. **Il ne faut pas effectuer l'étape trois** mais plutôt effectuer la manoeuvre suivante depuis le dossier principal du projet :

```
cd projet/frontend
composer install
cp .env.example .env
php artisan key:generate
php artisan passport:install
npm install
npm run prod
```

L'étape six du tutoriel ci-dessus n'est pas à effectuer non plus.

Il reste maintenant à régler le fichier de configuration de la frontend Inari. Le fichier `.env` du dossier `frontend` contient les clés nécessaires au fonctionnement de l'application. Il faut changer `DB_HOST`, `DB_DATABASE`, `DB_USERNAME` et `DB_PASSWORD` en mettant respectivement l'IP du serveur MySQL, le nom, le nom d'utilisateur et le mot de passe de la base de données.

Une fois cela fait, on peut exporter la base de données en utilisant la commande `php artisan migrate`.

Le serveur web est déployé. Il est nécessaire de démarrer le serveur Inari. Il suffit pour cela de se rendre dans le dossier `projet/agent` et de lancer les commandes suivantes :

`python3.7 -m pip install -r requirements.txt`

Il ensuite possible de démarrer le serveur Inari en lançant la commande suivante depuis le dossier `projet/agent` : `python3.7 src/start.py --server`. `--server` permet de démarrer le serveur.

Si Inari est lancé par SSH, il est recommandé de lancer Inari via `screen`. Pour cela, il suffit de taper la commande `screen`, puis de suivre les instructions du paragraphe précédent.

\pagebreak
\newappendix{Installation du module complémentaire Inari Express}

Inari Express est un module complémentaire, ou plug-in permettant de créer rapidement des clicks conformément au format attendu dans l'interface utilisateur Inari.

L'installer est très simple :

* Démarrez Firefox (version 42 minimum);
* Rendez-vous dans _modules complémentaires_;
* Cliquez sur l'engrenage en haut à droite et sélectionnez _installer un module depuis un fichier..._
* Sélectionnez le fichier `inari_express-1.0-signed.xpi` dans le dossier `projet/bachelor`.
* Acceptez la demande d'ajout de module complémentaire.

Vous avez installé Inari Express. Vous pouvez vous en servir en cliquant sur l'icône en haut à droite de Firefox. Pour exporter, cliquez droit n'importe où dans la page courante et sélectionnez _exporter_. Vos clicks sont désormais dans le presse-papier. Collez-les dans l'interface web Inari, sous _clicks_.

\pagebreak
\newappendix{Fichier de configuration Inari}

Cette annexe explique à quoi correspond chaque ligne du fichier de configuration Inari, le fichier `.env`.

* `CLIENT_WORKERS` : le nombre de workflow exécutables simultanément. Réglage conseillé : nombre de threads du processeur moins deux.
* `SERVER_WORKERS` : le nombre de workflow envoyables simultanément. Réglage conseillé : au moins quatre.
* `HTTPHANDLER_WEBSERVER_PORT` : le port du mini-serveur web de gestion des clients. Laisser à 7070.
* `CLIENT_PORT` : le port de communication du client. Laisser à 18965.
* `SERVER_PORT` : le port de communication du serveur. Laisser à 18964.
* `SERVER_WEB_FQDN` : Le (+FQDN_a) de l'interface web.
* `FILE_LOCKER` : Le chemin absolu du dossier accueillant les fichiers collectés par les workflow. Attention : il est indispensable que ce chemin absolu finisse avec un slash.
* `RESOURCES_PATH` : Le chemin absolu contenant les resources du projet. Si le dossier Inari est dans `/tmp/inari`, il faut indiquer `/tmp/inari/projet/agent/resources/`. A nouveau, le slash de fin est indispensable au bon fonctionnement.
